/* ****************************************************************************
 *  AnsiCGeneration.cpp
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.6 $
 *  $Date: 2002/12/14 20:52:16 $
 *  $Author: jhubbard $
 *************************************************************************** */
#include <string>
#include <iostream>
#include "AnsiCGeneration.h"
#include "Guidelines.h"

namespace code_sketch
{
using std::endl;
///////////////////////////////////////////////////////////////////////////////
// class AnsiCBodyGenerator
AnsiCBodyGenerator::AnsiCBodyGenerator( const Guidelines& arGuidelines )
	:	GeneratorBase( arGuidelines ),
		mFileCreator (arGuidelines.ClassBodyName(),
					  arGuidelines.OutputDirectory(),
					  arGuidelines.FileCreationFlags())
{
}

AnsiCBodyGenerator::~AnsiCBodyGenerator()
{
}

void AnsiCBodyGenerator::CreateProlog()
{
	SendToFile() <<
		"#include <stdio.h>"   															<< endl <<
		"#include \"" << GetGuidelines().ClassHeaderName() << "\""  					<< endl <<
		""   																			<< endl <<
		"int ExampleHelloRoutine()"														<< endl <<
		"{" 																			<< endl <<
		"    printf(\"Hello, from " << GetGuidelines().FirstClassName() <<
		               "!\\n\");" << endl <<
		"    return 0;" 																<< endl <<
		"}" 																			<< endl <<
		"" 																				<< endl;
}

///////////////////////////////////////////////////////////////////////////////
// class AnsiCHeaderGenerator
AnsiCHeaderGenerator::AnsiCHeaderGenerator( const Guidelines& arGuidelines )
	:	GeneratorBase( arGuidelines ),
		mFileCreator (arGuidelines.ClassHeaderName(),
					  arGuidelines.OutputDirectory(),
					  arGuidelines.FileCreationFlags())
{
}

AnsiCHeaderGenerator::~AnsiCHeaderGenerator()
{
}

void AnsiCHeaderGenerator::CreateProlog()
{
	SendToFile() <<
		"#ifndef " << MangledUpperCaseName( GetGuidelines() ) 	<< endl <<
		"#define " << MangledUpperCaseName( GetGuidelines() ) 	<< endl <<
		"" 														<< endl <<
		"#ifdef __cplusplus" 									<< endl <<
		"extern \"C\"" 											<< endl <<
		"{"														<< endl <<
		"#endif //  __cplusplus" 								<< endl <<
		"" 														<< endl;
}
void AnsiCHeaderGenerator::CreateBody(const std::string& arClassName)
{
	SendToFile() <<
		"    int ExampleHelloRoutine();" 						<< endl <<
		"" 														<< endl;
}

void AnsiCHeaderGenerator::CreateEpilog()
{
	SendToFile() <<
		"#ifdef __cplusplus" 												<< endl <<
		"}"																	<< endl <<
		"#endif //  __cplusplus" 											<< endl <<
		""																	<< endl <<
		"#endif // " << MangledUpperCaseName( GetGuidelines() ) 			<< endl <<
		"" 																	<< endl;
}

} // end of the code_sketch namespace
