/* ****************************************************************************
 *  AnsiCGeneration.h
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.6 $
 *  $Date: 2002/12/14 20:52:16 $
 *  $Author: jhubbard $
 *************************************************************************** */
#ifndef JFH_CODE_SKETCH_ANSICGENERATION_H
#define JFH_CODE_SKETCH_ANSICGENERATION_H

#include "GeneratorBase.h"
#include "FileCreator.h"
#include <fstream>

namespace code_sketch
{
/** Purpose: Encapsulate the logic for creating a body file for ANSI C code.
 */
class AnsiCBodyGenerator : public GeneratorBase
{
public:
    /// Constructor.
    AnsiCBodyGenerator( const Guidelines& arGuidelines );
    virtual ~AnsiCBodyGenerator();
	virtual void CreateProlog();
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    AnsiCBodyGenerator(const AnsiCBodyGenerator&);
    AnsiCBodyGenerator& operator=(const AnsiCBodyGenerator&);

	FileCreator		mFileCreator;
};
/** Purpose: Encapsulate the logic for creating a header file for ANSI C code.
 */
class AnsiCHeaderGenerator : public GeneratorBase
{
public:
    /// Constructor.
    AnsiCHeaderGenerator( const Guidelines& arGuidelines );
    virtual ~AnsiCHeaderGenerator();
	virtual void CreateProlog();
	virtual void CreateBody(const std::string& arClassName) ;
	virtual void CreateEpilog();
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    AnsiCHeaderGenerator(const AnsiCHeaderGenerator&);
    AnsiCHeaderGenerator& operator=(const AnsiCHeaderGenerator&);

	FileCreator		mFileCreator;
};
} // end of the code_sketch namespace

#endif // #ifndef JFH_CODE_SKETCH_ANSICGENERATION_H

