/* ****************************************************************************
 *  CodeSketchMain.cpp
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.14 $
 *  $Date: 2002/12/16 21:00:25 $
 *  $Author: jhubbard $
 *************************************************************************** */
#include <string>
#include <iostream>
#include "Guidelines.h"
#include "CodeSketch.h"

void Usage();
void ReportVersion();

int main(int argc, char* argv[])
{
	try
	{
		code_sketch::Guidelines guidelines(argc, argv);
		if( guidelines.HelpRequested()
			|| guidelines.Count() == 0)
		{
			Usage();
			exit(0);
		}
		else if( guidelines.VersionReportRequested())
		{
			ReportVersion();
			exit(0);
		}
		else if( guidelines.DiagnosticsReportRequested() )
		{
			guidelines.DumpDiagnostics();
			exit(0);
		}
		else
		{
			code_sketch::CodeSketch codeSketcher(guidelines);
			codeSketcher.Run();
		}
	}
	catch(std::string& commandLineError)
	{
		std::cout << "CodeSketch input error: " << commandLineError << std::endl;
		Usage();
		exit(1);
	}
    return 0;
}

void ReportVersion()
{
	std::cout <<
	"This is the CodeSketch program, version 1.503, created on 14 Dec 2002." << std::endl;
}


void Usage()
{
	using std::endl;
	std::cout <<
	"" << endl <<
	"---------------------------------------------------------------------------" << endl;
	ReportVersion();
	std::cout <<
	"There are four modes of usage: C++ class generator, ANSI C generator,"   	<< endl <<
	"and device driver generator, and diagnostics."								<< endl <<
	""	 																		<< endl <<
	"Usage 1: C++ class generator (with optional makefile and unit test)" 		<< endl <<
	"         CodeSketch -class \"class1 class2 ... classN\""					<< endl <<
	"          [ -namespace name | -ns name ]"									<< endl <<
	"          [ -interface | -i ] (creates a pure virtual (interface) class" 	<< endl <<
	"          [ -no_dump_diagnostics | -no_dd ]"  								<< endl <<
	"          [ -no_icc ] (suppresses generation of the .icc file)"			<< endl <<
	"          [ -base_filename filename | -file filename ]"					<< endl <<
	"          ...other options (see common options, below)"					<< endl <<
	""	 										 				   				<< endl <<
	"Usage 2: ANSI C generator: " 												<< endl <<
	"         CodeSketch -ansi_c -class someName"    							<< endl <<
	"          ...other options (see common options, below)"					<< endl <<
	""	 																		<< endl <<
	"Usage 3: Device Driver generator (with optional makefile or bash build script)"  << endl <<
	"         CodeSketch -class driverName"											<< endl <<
	"             -os operatingSystem        (may be lynx or linux) " 				<< endl <<
	"             -char_driver  | -char "											<< endl <<
	"             [ -block_driver | -block ] (not yet supported)" 				  	<< endl <<
	"             [ -sparse ]                (suppress generation of shared files)"	<< endl <<
	"          ...other options (see common options, below)"						<< endl <<
	""	 										    								<< endl <<
	"Usage 4: Diagnostics (no files are generated)"									<< endl <<
	"         CodeSketch -class \"class1 class2 ... classN\"" 						<< endl <<
	"                  [ -diagnostics_only | -diag ]" 								<< endl <<
	"          ...other options (see common options, below)"				   		<< endl <<
	"    or:"									    								<< endl <<
	"         CodeSketch -version | -v    (Reports version and exits)"		 		<< endl <<
	"         CodeSketch -help    | -h    (Prints this screen and exits)"		 	<< endl <<
	""	 										    								<< endl <<
	"Common optional parameters: (most common are listed first)"					<< endl <<
	"          [ -author your_name ]"					 							<< endl <<
	"          [ -initials your_initials     | -in your_initials ]"					<< endl <<
	"          [ -copyright copyright_string | -c copyright_string ]"  		  		<< endl <<
	"          [ -output_dir a_directory     | -o a_directory ]"					<< endl <<
	"          [ -open_source                | -open ]"  	  	  					<< endl <<
	"          [ -overwrite                  | -ow ]"					 			<< endl <<
	"          [ -quiet                      | -q ]"					   			<< endl <<
	"          [ -no_sccs_keywords           | -no_sccs ]"  	   					<< endl <<
	"          [ -no_unit_test               | -no_ut ]" 							<< endl <<
	"          [ -no_makefile                | -no_mf ]"	  					   	<< endl <<
	""	 																			<< endl <<
	"For comprehensive examples, please see BasicTest.sh, included with CodeSketch." << endl <<
	""	 																			<< endl <<
	"Created by John F. Hubbard.  Notes, feedback, and software updates are " 		<< endl <<
	"maintained here: http://www.hubbard-software.com" 								<< endl <<
	"---------------------------------------------------------------------------" << endl;
}
