#!/bin/bash
#############################################################################
#  CommonBuildRoutines.bash
#  Created by George Jones, on Sat Dec 14 2002, 23:56:01 PST
#
#  Copyright (C) 3001, G.Jones
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License (LGPL) along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#  $Revision: 1.1 $
#  $Date: 2002/12/15 09:23:13 $
#  $Author: jhubbard $
#############################################################################

# Usage: Source this file from within your specific build script.
#
#      -tidy removes the object files from the driver directory,
#      so as to make it easier to type the insmod command (for example).
#
#      -verbose shows more details as to what compile and link commands
#      are being issued.
#
###############################################################################

echo "Performing a clean build of the following drivers: ${PROGRAMS}"

if [ ${TARGET} = "LINUX_X86" ];then
    CC=gcc
    LINK="ld -r"
    KERNELDIR="/usr/src/linux-2.4"
    INCLUDES="-I ${KERNELDIR}/include -I . "

    CFLAGS="-D __KERNEL__  -D MODULE -O2 -Wall -Wno-unknown-pragmas -Wno-unused"

elif [ ${TARGET} = "LYNXOS_PPC" ];then
    CC=gcc
    LINK="ld -bM:SRE -bimport:${LYNXOS_KERNEL_SYMBOLS_IMPORT_FILENAME}"
    INCLUDES=""
    INCLUDES="${INCLUDES} -I/usr/lynx/3.1.0/ppc/sys/include/kernel"
    INCLUDES="${INCLUDES} -I/usr/lynx/3.1.0/ppc/sys/include/family/ppc"
    INCLUDES="${INCLUDES} -I/usr/lynx/3.1.0/ppc/sys/bsp.pp_drm"

    CFLAGS="-D${TARGET} -O2 -Wall -Wno-unknown-pragmas -Wno-unused -D__NO_INCLUDE_WARN__ -D__LYNXOS"
else
    echo "ERROR: TARGET=${TARGET}, but only LINUX_X86 and LYNXOS_PPC are supported."
    exit -1
fi

if [ -d ${TARGET} ];then rm -r ${TARGET}; fi
if [ ! -d ${TARGET} ];then mkdir ${TARGET}; fi

# Search input parameters for position-independent switches:
VERBOSE="false"
TIDY="false"

for ARGS in $*
do
    if [ ${ARGS} = "-tidy" ] ;  then TIDY="true";    fi
    if [ ${ARGS} = "-verbose" ];then VERBOSE="true"; fi
done

function compile_files
{
    local sourceFile
    for sourceFile in $*
    do
        echo "Compiling ${sourceFile} ..."
        MakeObjectFileFromSourceFile ${sourceFile}

        if [ ${VERBOSE} = "true" ]; then
            echo "${CC} -c ${CFLAGS} ${INCLUDES} ${sourceFile} -o ${OBJ_FILE}"
        fi

        ${CC} -c ${CFLAGS} ${INCLUDES} ${sourceFile} -o ${OBJ_FILE}
    done
}

function MakeObjectFileFromSourceFile
{
    local sourceFile=$@
    # Strip off any reference to another directory (such as ../otherDir/)
    # This has the effect of requiring unique source file names, even
    # though they may reside in different directories.
    sourceFile=${sourceFile##*/}
    objFile=${sourceFile%.*}.o
    OBJ_FILE="${TARGET}/${objFile}"
}

function MakeObjectFileListFromSourceFileList
{
    # OBJ_FILE_LIST is not local; function result is placed in this variable.
    OBJ_FILE_LIST=""
    for sourceFile in $*
    do
        MakeObjectFileFromSourceFile ${sourceFile}
        OBJ_FILE_LIST="${OBJ_FILE_LIST} ${OBJ_FILE}"
    done
}

function MakeSourceFileListFromProgramName
{
    # SOURCE_FILE_LIST is not local; function result is placed in this variable.
    SOURCE_FILE_LIST=""
    local programName=$1
    local temp="${programName}_FILES"
    eval SOURCE_FILE_LIST=$(echo "\${${temp}}")
}

function link_files
{
    local programName=$1
    local outputFile=${TARGET}/${programName}.drv
    echo "Linking ${outputFile}..."
    MakeSourceFileListFromProgramName ${programName}

    MakeObjectFileListFromSourceFileList ${SOURCE_FILE_LIST} ${COMMON_FILES}

    if [ ${VERBOSE} = "true" ]; then
        echo "${LINK} ${OBJ_FILE_LIST} -o ${outputFile}"
    fi
    ${LINK} ${OBJ_FILE_LIST} -o ${outputFile}
}

# Compile files that are unique to each program:
for programName in ${PROGRAMS}
do
    MakeSourceFileListFromProgramName ${programName}
    compile_files ${SOURCE_FILE_LIST}
done

# Compile files that shared among two or more programs:
compile_files ${COMMON_FILES}

# Link each program:
for name in ${PROGRAMS}
do
    link_files ${name}
done

if [ ${TIDY} = "true" ]; then
    echo "Removing intermediate files, because you specified -tidy..."
    rm ${TARGET}/*.o
fi
