/* ****************************************************************************
 *  CplusplusGeneration.cpp
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.13 $
 *  $Date: 2002/12/15 05:57:26 $
 *  $Author: jhubbard $
 *************************************************************************** */
#include <string>
#include <iostream>
#include "CplusplusGeneration.h"
#include "Guidelines.h"

namespace code_sketch
{
using std::endl;
///////////////////////////////////////////////////////////////////////////////
// class ClassHeaderGenerator

ClassHeaderGenerator::ClassHeaderGenerator( const Guidelines& arGuidelines )
	:	GeneratorBase( arGuidelines ),
		mFileCreator (arGuidelines.ClassHeaderName(),
					  arGuidelines.OutputDirectory(),
					  arGuidelines.FileCreationFlags())
{
}

ClassHeaderGenerator::~ClassHeaderGenerator()
{
}

void ClassHeaderGenerator::CreateProlog()
{
	SendToFile() <<
		"#ifndef " << MangledUpperCaseName(GetGuidelines()) << endl <<
		"#define " << MangledUpperCaseName(GetGuidelines()) << endl <<
		""						<< endl;
	if( GetGuidelines().NamespaceDesired() )
	{
		SendToFile() <<
			"namespace " << GetGuidelines().Namespace() << endl <<
			"{" << endl;
	}
}

void ClassHeaderGenerator::CreateBody(const std::string& arClassName)
{
	SendToFile() <<
		"    /** TODO: Describe the " << arClassName <<
								" class in this doc++ comment block." << endl <<
		"     *  " << endl <<
		"     */" << endl <<
		"    class " << arClassName << endl <<
		"    {" << endl <<
		"    public:" << endl;

	if( GetGuidelines().PureInterfaceClassDesired() )
	{
		SendToFile() <<
			"        virtual ~" << arClassName << "() {}" << endl <<
			"        virtual void ExampleInterfaceMethod() = 0;" << endl;
	}
	else
	{
		SendToFile() <<
			"        " << arClassName << "();" << endl <<
			"        virtual ~" << arClassName << "();" << endl <<
			"" << endl;

		if( GetGuidelines().DumpDiagnosticsDesired() )
		{
			SendToFile() <<
				"        /// DumpDiagnostics sends the object\'s state to stdout."  << endl <<
				"        void DumpDiagnostics() const;"  << endl;
		}
		SendToFile() <<
			""  << endl <<
			"    private:" << endl <<
			"        // Copying or assignment of this class is prohibited, at this point:" << endl <<
			"        " << arClassName << "(const " << arClassName << "&);" << endl <<
			"        " << arClassName << "& operator=(const " << arClassName << "&);" << endl;
	}
	SendToFile() <<
		"    };"  << endl <<
		""  << endl;
}

void ClassHeaderGenerator::CreateEpilog()
{
	if( GetGuidelines().NamespaceDesired() )
	{
		SendToFile() <<
			"} // end of namespace " << GetGuidelines().Namespace() << endl;
	}
	if(      GetGuidelines().InlineFileDesired()
		&& ! GetGuidelines().PureInterfaceClassDesired() )
	{
		SendToFile() <<
			"#include \"" << GetGuidelines().ClassInlineName() << "\""<< endl <<
			""<< endl;
	}
	SendToFile() <<
		"#endif // " << MangledUpperCaseName(GetGuidelines()) << endl;
}

///////////////////////////////////////////////////////////////////////////////
// class ClassInlineGenerator

ClassInlineGenerator::ClassInlineGenerator( const Guidelines& arGuidelines )
	:	GeneratorBase( arGuidelines ),
		mFileCreator (arGuidelines.ClassInlineName(),
					  arGuidelines.OutputDirectory(),
					  arGuidelines.FileCreationFlags())
{

}

ClassInlineGenerator::~ClassInlineGenerator()
{

}
void ClassInlineGenerator::CreateProlog()
{
	if( GetGuidelines().NamespaceDesired() )
	{
		SendToFile() <<
			"namespace " << GetGuidelines().Namespace() << endl <<
			"{" << endl;
	}
}

void ClassInlineGenerator::CreateBody(const std::string& arClassName)
{
}

void ClassInlineGenerator::CreateEpilog()
{
	if( GetGuidelines().NamespaceDesired() )
	{
		SendToFile() <<
			"} // end of namespace " << GetGuidelines().Namespace() << endl;
	}
}
///////////////////////////////////////////////////////////////////////////////
// class ClassBodyGenerator

ClassBodyGenerator::ClassBodyGenerator( const Guidelines& arGuidelines )
	:	GeneratorBase( arGuidelines ),
		mFileCreator (arGuidelines.ClassBodyName(),
					  arGuidelines.OutputDirectory(),
					  arGuidelines.FileCreationFlags())
{

}

ClassBodyGenerator::~ClassBodyGenerator()
{

}

void ClassBodyGenerator::CreateProlog()
{
	SendToFile() <<
		"#include <string>" << endl <<
		"#include <iostream>" << endl <<
		"#include \"" << GetGuidelines().ClassHeaderName() << "\""  	<< endl <<
		""						<< endl;
	if( GetGuidelines().NamespaceDesired() )
	{
		SendToFile() <<
			"namespace " << GetGuidelines().Namespace() << endl <<
			"{" << endl;
	}
}

void ClassBodyGenerator::CreateBody(const std::string& arClassName)
{
	SendToFile() <<
		"///////////////////////////////////////"
		"///////////////////////////////////////"  << endl <<
		"// class " << arClassName 							<< endl <<
		"" << arClassName << "::" << arClassName << "()"	<< endl <<
		"{" 												<< endl <<
		"}" 												<< endl <<	
		"" << arClassName << "::~" << arClassName << "()"	<< endl <<
		"{" 												<< endl <<
		"}" 												<< endl;

		if( GetGuidelines().DumpDiagnosticsDesired())
		{
			SendToFile() <<
				"void " << arClassName << "::DumpDiagnostics() const"  << endl <<
				"{"  << endl <<
				"    std::cout << \"" << arClassName <<
				                      " diagnostics dump.\" << std::endl;"  << endl <<
				"}"  << endl;
		}
}

void ClassBodyGenerator::CreateEpilog()
{
	if( GetGuidelines().NamespaceDesired() )
	{
		SendToFile() <<
			"" << endl <<
			"} // end of namespace " << GetGuidelines().Namespace() << endl;
	}
}
///////////////////////////////////////////////////////////////////////////////
// class UnitTestGenerator

UnitTestGenerator::UnitTestGenerator( const Guidelines& arGuidelines )
	:	GeneratorBase( arGuidelines ),
		mFileCreator (arGuidelines.UnitTestName(),
					  arGuidelines.OutputDirectory(),
					  arGuidelines.FileCreationFlags())
{

}

UnitTestGenerator::~UnitTestGenerator()
{

}

void UnitTestGenerator::CreateProlog()
{
	if( GetGuidelines().ModeIsCplusplus() )
	{
		SendToFile() <<
			"#include <iostream>"  << endl;
	}
	else if( GetGuidelines().ModeIsAnsiC() )
	{
		SendToFile() <<
			"#include <stdio.h>"  << endl;
	}
	SendToFile() <<
		"#include \"" << GetGuidelines().ClassHeaderName() << "\""  	<< endl <<
		""   														<< endl;
	SendToFile() <<
		"int main(int argc, char * argv[])" << endl <<
		"{" 								<< endl;

	if( GetGuidelines().ModeIsCplusplus() )
	{
		if( GetGuidelines().NamespaceDesired() )
		{
			SendToFile() <<
				"    using namespace " << GetGuidelines().Namespace() << ";" << endl;
		}
	}
}
void UnitTestGenerator::CreateBody(const std::string& arClassName)
{
	if( GetGuidelines().ModeIsCplusplus() )
	{
		if( GetGuidelines().PureInterfaceClassDesired() )
		{
			SendToFile() <<
				"    class " << arClassName << "Child : public " << arClassName << "" << endl <<
				"    {" << endl <<
				"    public:" << endl <<
				"	    void ExampleInterfaceMethod()" << endl <<
				"	    {" << endl <<
				"		    std::cout << \"Hello from the concrete implementation of the "
													<< arClassName << " interface!\"" << endl <<
				"			    	  << std::endl;" << endl <<
				"	    }" << endl <<
				"    };" << endl <<
				"" << endl <<
				"    " << arClassName << "Child instanceOf" << arClassName << "Child;" << endl <<
				"    instanceOf" << arClassName << "Child.ExampleInterfaceMethod();" << endl;
		}
		else // Normal class (not pure virtual)
		{
			SendToFile() <<
				"    " << arClassName << " instanceOf" << arClassName << ";" << endl;

			if( GetGuidelines().DumpDiagnosticsDesired())
			{
				SendToFile() <<
					"    instanceOf" << arClassName << ".DumpDiagnostics();" << endl <<
					"" << endl;
			}
		}
	}
	else if( GetGuidelines().ModeIsAnsiC() )
	{
		SendToFile() <<
			"    ExampleHelloRoutine();" << endl;
	}
}

void UnitTestGenerator::CreateEpilog()
{
		
	SendToFile() <<
		"    return 0;"			<< endl <<
		"}"						<< endl <<
		"" 						<< endl;
}
///////////////////////////////////////////////////////////////////////////////
// class MakefileGenerator

MakefileGenerator::MakefileGenerator( const Guidelines& arGuidelines )
	:	GeneratorBase( arGuidelines ),
		mFileCreator (arGuidelines.MakefileName(),
					  arGuidelines.OutputDirectory(),
					  arGuidelines.FileCreationFlags())
{
}

MakefileGenerator::~MakefileGenerator()
{
}

void MakefileGenerator::CreateHeader()
{
	CreateStandardScriptHeader( GetGuidelines().MakefileName(), "" );
}

void MakefileGenerator::CreateProlog()
{
	if( GetGuidelines().ModeIsDeviceDriver() )
	{
		SendToFile() <<
			"" << endl <<
			"all: tidy" << endl <<
			"" << endl <<
			"debug: verbose" << endl <<
			"" << endl <<
			"setup: " << endl <<
			"\tif [ ! -x " << GetGuidelines().DeviceDriverScriptName()
				<< " ]; then chmod u+x " << GetGuidelines().DeviceDriverScriptName()
				<< "; fi" << endl <<
			"\tif [ ! -d ${TARGET} ]; then mkdir ${TARGET};fi" << endl <<
			"" << endl <<
			"verbose: setup" << endl <<
			"\t./" << GetGuidelines().DeviceDriverScriptName() << "  -verbose" << endl <<
			"" << endl <<
			"tidy: setup" << endl <<
			"\t./" << GetGuidelines().DeviceDriverScriptName() << "  -tidy" << endl <<
			"" << endl <<
			"lynx_info: setup ${TARGET}/" << GetGuidelines().FirstClassName() << "_CreateInfo" << endl <<
			"\t${TARGET}/" << GetGuidelines().FirstClassName() << "_CreateInfo > ${TARGET}/" << GetGuidelines().FirstClassName() << "_Info.bin" << endl <<
			"	" << endl <<
			"${TARGET}/" << GetGuidelines().FirstClassName() << "_CreateInfo: " << GetGuidelines().FirstClassName() << "_CreateInfo.c" << endl <<
			"\tgcc $< -o $@" << endl <<
			"" << endl;
	}
	else
	{
		CreateStandardMakefile();
	}
}

} // end of the code_sketch namespace
