/* ****************************************************************************
 *  DeviceDriverGeneration.cpp
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.22 $
 *  $Date: 2002/12/16 21:00:25 $
 *  $Author: jhubbard $
 *************************************************************************** */
#include <string>
#include <iostream>
#include "DeviceDriverGeneration.h"
#include "Guidelines.h"

namespace code_sketch
{
using std::endl;
///////////////////////////////////////////////////////////////////////////////
// class Linux_x86_DeviceDriverGenerator

Linux_x86_DeviceDriverGenerator::Linux_x86_DeviceDriverGenerator( const Guidelines& arGuidelines )
    :    GeneratorBase( arGuidelines ),
        mFileCreator (arGuidelines.ClassBodyName(),
                      arGuidelines.OutputDirectory(),
                      arGuidelines.FileCreationFlags())
{
}

Linux_x86_DeviceDriverGenerator::~Linux_x86_DeviceDriverGenerator()
{
}

void Linux_x86_DeviceDriverGenerator::CreateBody(const std::string& arClassName)
{
    SendToFile() <<
    "#ifdef __KERNEL__" << endl <<
    "    #include <linux/types.h>" << endl <<
    "    #include <linux/stddef.h>" << endl <<
    "    #include <linux/kernel.h>" << endl <<
    "    #include <linux/config.h>" << endl <<
    "    #include <linux/module.h>" << endl <<
    "    #include <linux/pci.h>" << endl <<
    "    #include <linux/fs.h>" << endl <<
    "    #include <asm/uaccess.h>" << endl <<
    "    #include <asm/semaphore.h>" << endl <<
    "#else" << endl <<
    "     #error \"Failed to define __KERNEL__\"" << endl <<
    "#endif" << endl <<
    "" << endl <<
    "#include \"StandardCodeSketchCommands.h\"" << endl <<
    "" << endl <<
    "#ifdef MODULE" << endl <<
    "    MODULE_LICENSE(\"GPL and additional rights\");" << endl <<
    "    MODULE_AUTHOR(\"" << GetGuidelines().Author() << "\");" << endl <<
    "    MODULE_DESCRIPTION(\"" << GetGuidelines().FirstClassName() << ": A full prototype device driver.\");" << endl <<
    "#else" << endl <<
    "     #error \"Failed to define MODULE\"" << endl <<
    "#endif" << endl <<
    "" << endl <<
    "#define MAX_FULL_DRIVER_STORAGE          512" << endl <<
    "#define MAX_DIAG_MESSAGE_LENGTH          512" << endl <<
    "" << endl <<
    "loff_t             " << GetGuidelines().FirstClassName() << "_llseek(struct file * apFile, loff_t aOffset, int whence);" << endl <<
    "static int         " << GetGuidelines().FirstClassName() << "_open(struct inode *, struct file *);" << endl <<
    "static int         " << GetGuidelines().FirstClassName() << "_release(struct inode *, struct file *);" << endl <<
    "static ssize_t     " << GetGuidelines().FirstClassName() << "_read(struct file *, char *, size_t, loff_t *);" << endl <<
    "static ssize_t     " << GetGuidelines().FirstClassName() << "_write(struct file *, const char *, size_t, loff_t *);" << endl <<
    "static int         " << GetGuidelines().FirstClassName() << "_ioctl( struct inode *, struct file *, unsigned int, unsigned long );" << endl <<
    "" << endl <<
    "// Linux kernel 2.4.18-10 file operations signatures, for reference:" << endl <<
    "// struct file_operations {" << endl <<
    "//     struct module *owner;" << endl <<
    "//     loff_t (*llseek) (struct file *, loff_t, int);" << endl <<
    "//     ssize_t (*read) (struct file *, char *, size_t, loff_t *);" << endl <<
    "//     ssize_t (*write) (struct file *, const char *, size_t, loff_t *);" << endl <<
    "//     int (*readdir) (struct file *, void *, filldir_t);" << endl <<
    "//     unsigned int (*poll) (struct file *, struct poll_table_struct *);" << endl <<
    "//     int (*ioctl) (struct inode *, struct file *, unsigned int, unsigned long);" << endl <<
    "//     int (*mmap) (struct file *, struct vm_area_struct *);" << endl <<
    "//     int (*open) (struct inode *, struct file *);" << endl <<
    "//     int (*flush) (struct file *);" << endl <<
    "//     int (*release) (struct inode *, struct file *);" << endl <<
    "//     int (*fsync) (struct file *, struct dentry *, int datasync);" << endl <<
    "//     int (*fasync) (int, struct file *, int);" << endl <<
    "//     int (*lock) (struct file *, int, struct file_lock *);" << endl <<
    "//     ssize_t (*readv) (struct file *, const struct iovec *, unsigned long, loff_t *);" << endl <<
    "//     ssize_t (*writev) (struct file *, const struct iovec *, unsigned long, loff_t *);" << endl <<
    "//     ssize_t (*sendpage) (struct file *, struct page *, int, size_t, loff_t *, int);" << endl <<
    "//     unsigned long (*get_unmapped_area)(struct file *, unsigned long, unsigned long, unsigned long, unsigned long);" << endl <<
    "// };" << endl <<
    "" << endl <<
    "struct file_operations Fops =" << endl <<
    "{" << endl <<
    "    llseek:  " << GetGuidelines().FirstClassName() << "_llseek,    " << endl <<
    "    read:    " << GetGuidelines().FirstClassName() << "_read,    " << endl <<
    "    write:   " << GetGuidelines().FirstClassName() << "_write,     " << endl <<
    "    ioctl:   " << GetGuidelines().FirstClassName() << "_ioctl,    " << endl <<
    "    open:    " << GetGuidelines().FirstClassName() << "_open,    " << endl <<
    "    release: " << GetGuidelines().FirstClassName() << "_release" << endl <<
    "};" << endl <<
    "" << endl <<
    "typedef struct " << GetGuidelines().FirstClassName() << "Data" << endl <<
    "{" << endl <<
    "    loff_t     mSize;" << endl <<
    "    loff_t     mOffset;" << endl <<
    "    char       mpBuffer[MAX_FULL_DRIVER_STORAGE];" << endl <<
    "    char       mpDiagnosticsMessage[MAX_DIAG_MESSAGE_LENGTH];" << endl <<
    "" << endl <<
    "} " << GetGuidelines().FirstClassName() << "Data;" << endl <<
    "" << endl <<
    "" << endl <<
    "static const int          SUCCESS     = 0;" << endl <<
    "static const char * const DEVICE_NAME = \"" << GetGuidelines().FirstClassName() << "\";" << endl <<
    "" << endl <<
    "static int  gMajorDeviceNumber        = 0;" << endl <<
    "static struct semaphore gSem;" << endl <<
    "static " << GetGuidelines().FirstClassName() << "Data * gpDEVICE =0;" << endl <<
    "" << endl <<
    "" << endl <<
    "int" << endl <<
    "init_module()" << endl <<
    "{" << endl <<
    "    const char * pMessage = \"This data was written when init_module() was called.\\n\";" << endl <<
    "    printk(KERN_INFO \"init_module: Allocating resources.\\n\");" << endl <<
    "    // Allocate and initialize the private data used by this device:" << endl <<
    "    gpDEVICE = (" << GetGuidelines().FirstClassName() << "Data *) kmalloc(sizeof(" << GetGuidelines().FirstClassName() << "Data), GFP_KERNEL);" << endl <<
    "" << endl <<
    "    if( !gpDEVICE )" << endl <<
    "    {   return -ENOMEM;" << endl <<
    "    }" << endl <<
    "" << endl <<
    "    memset(gpDEVICE, '\\0', sizeof(*gpDEVICE) );" << endl <<
    "    strcpy(gpDEVICE->mpBuffer, pMessage);" << endl <<
    "    gpDEVICE->mSize = strlen(pMessage);" << endl <<
    "    strcpy(gpDEVICE->mpDiagnosticsMessage, \"Original diagnostics message; overwritten later\");" << endl <<
    "" << endl <<
    "    sema_init(&gSem, 1);" << endl <<
    "" << endl <<
    "    gMajorDeviceNumber = register_chrdev(0, DEVICE_NAME, &Fops);" << endl <<
    "" << endl <<
    "    if (gMajorDeviceNumber < 0)" << endl <<
    "    {" << endl <<
    "        printk (\"Failed to register %s, major number %d\\n\", DEVICE_NAME, gMajorDeviceNumber);" << endl <<
    "        return gMajorDeviceNumber;" << endl <<
    "    }" << endl <<
    "    " << endl <<
    "    printk (\"Successully registered %s, major number %d\\n\", DEVICE_NAME, gMajorDeviceNumber);" << endl <<
    "    printk (\"Run: mknod /dev/%s c %d 0\\n\", DEVICE_NAME, gMajorDeviceNumber);" << endl <<
    "    return 0;" << endl <<
    "}" << endl <<
    "" << endl <<
    "void" << endl <<
    "cleanup_module()" << endl <<
    "{" << endl <<
    "    int result = 0;" << endl <<
    "" << endl <<
    "    printk(KERN_INFO \"cleanup_module: Freeing resources.\\n\");" << endl <<
    "    kfree(gpDEVICE);" << endl <<
    "" << endl <<
    "" << endl <<
    "    result = unregister_chrdev(gMajorDeviceNumber, DEVICE_NAME);" << endl <<
    "    if (result < 0)" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"Failed to unregister the device driver, error code: %d\\n\", result);" << endl <<
    "        return;" << endl <<
    "    }" << endl <<
    "    printk (\"Successully unregistered %s\\n\", DEVICE_NAME);" << endl <<
    "}" << endl <<
    "" << endl <<
    "int" << endl <<
    "" << GetGuidelines().FirstClassName() << "_open(struct inode * apInode, struct file * apFile)" << endl <<
    "{" << endl <<
    "    " << GetGuidelines().FirstClassName() << "Data * pDevice = 0;" << endl <<
    "    if( !apFile )" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"ERROR in " << GetGuidelines().FirstClassName() << "_open: a NULL file * was provided.\\n\");" << endl <<
    "        return -EINVAL;" << endl <<
    "    }" << endl <<
    "" << endl <<
    "    if( ! apFile->private_data)" << endl <<
    "    {" << endl <<
    "        apFile->private_data = gpDEVICE;" << endl <<
    "    }" << endl <<
    "    " << endl <<
    "    MOD_INC_USE_COUNT;" << endl <<
    "    return 0;" << endl <<
    "}" << endl <<
    "" << endl <<
    "int" << endl <<
    "" << GetGuidelines().FirstClassName() << "_release(struct inode * apInode, struct file * apFile)" << endl <<
    "{" << endl <<
    "    " << GetGuidelines().FirstClassName() << "Data * pDevice = 0;" << endl <<
    "    if( !apFile || !apFile->private_data )" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"ERROR in " << GetGuidelines().FirstClassName() << "_release: NULL input detected.\\n\");" << endl <<
    "        return -EINVAL;" << endl <<
    "    }" << endl <<
    "" << endl <<
    "    if( apFile->private_data)" << endl <<
    "    {" << endl <<
    "        apFile->private_data = NULL;" << endl <<
    "    }" << endl <<
    "" << endl <<
    "    MOD_DEC_USE_COUNT;" << endl <<
    "    return 0;" << endl <<
    "}" << endl <<
    "" << endl <<
    "loff_t" << endl <<
    "" << GetGuidelines().FirstClassName() << "_llseek(struct file * apFile, loff_t aOffset, int whence)" << endl <<
    "{" << endl <<
    "    // return -ESPIPE; // This is how to disable llseek correctly." << endl <<
    "" << endl <<
    "    " << GetGuidelines().FirstClassName() << "Data * pDevice = 0;" << endl <<
    "    loff_t           newPosition = 0;" << endl <<
    "" << endl <<
    "    if( !apFile || !apFile->private_data )" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"ERROR in " << GetGuidelines().FirstClassName() << "_llseek: NULL input detected.\\n\");" << endl <<
    "        return -EINVAL;" << endl <<
    "    }" << endl <<
    "    pDevice = apFile->private_data;" << endl <<
    "" << endl <<
    "    switch(whence)" << endl <<
    "    {" << endl <<
    "    case 0: // SEEK_SET" << endl <<
    "        newPosition = aOffset;" << endl <<
    "        break;" << endl <<
    "" << endl <<
    "    case 1: // SEEK_CUR" << endl <<
    "        newPosition = pDevice->mOffset + aOffset;" << endl <<
    "        break;" << endl <<
    "" << endl <<
    "    case 2: // SEEK_END" << endl <<
    "        newPosition = pDevice->mSize + aOffset;" << endl <<
    "        break;" << endl <<
    "" << endl <<
    "    default:" << endl <<
    "        return -EINVAL;" << endl <<
    "    }" << endl <<
    "" << endl <<
    "    if( newPosition < 0 )" << endl <<
    "    {" << endl <<
    "        return -EINVAL;" << endl <<
    "    }" << endl <<
    "    pDevice->mOffset = newPosition;" << endl <<
    "    return newPosition;" << endl <<
    "}" << endl <<
    "" << endl <<
    "static int" << endl <<
    "" << GetGuidelines().FirstClassName() << "_ioctl( struct inode * apInode," << endl <<
    "                      struct file * apFile," << endl <<
    "                      unsigned int aCommand," << endl <<
    "                      unsigned long aArg)" << endl <<
    "{" << endl <<
    "    " << GetGuidelines().FirstClassName() << "Data * pDevice = 0;" << endl <<
    "    StandardCodeSketchCommands * pCommandData = 0;" << endl <<
    "    char exampleMessage[] = \"Example diagnostics message from within " << GetGuidelines().FirstClassName() << "_ioctl.\\n\";" << endl <<
    "" << endl <<
    "    if( !apFile || !apFile->private_data )" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"ERROR in " << GetGuidelines().FirstClassName() << "_ioctl: NULL input detected.\\n\");" << endl <<
    "        return -EINVAL;" << endl <<
    "    }" << endl <<
    "    pDevice = apFile->private_data;" << endl <<
    "" << endl <<
    "    if( CODE_SKETCH_OVERALL_COMMAND_TO_IOCTL == aCommand )" << endl <<
    "    {" << endl <<
    "        if( 0 == aArg )" << endl <<
    "        {" << endl <<
    "            return -EINVAL;" << endl <<
    "        }" << endl <<
    "" << endl <<
    "        pCommandData = (StandardCodeSketchCommands *) aArg;" << endl <<
    "" << endl <<
    "        if (down_interruptible(&gSem))" << endl <<
    "        {" << endl <<
    "            printk(KERN_INFO \"Error in " << GetGuidelines().FirstClassName() << "_write, down_interruptible call.\\n\");" << endl <<
    "            return -ERESTARTSYS;" << endl <<
    "        }" << endl <<
    "        " << endl <<
    "        switch( pCommandData->mCommand )" << endl <<
    "        {" << endl <<
    "        case CODE_SKETCH_COMMAND_DUMP:" << endl <<
    "" << endl <<
    "            sprintf(pDevice->mpDiagnosticsMessage, \"Diagnostics report: \\n\"" << endl <<
    "                    \"Size: %llu, Offset (seek position): %llu \\n\"," << endl <<
    "                    pDevice->mSize, pDevice->mOffset);" << endl <<
    "            if (copy_to_user(pCommandData->mBufferOut, pDevice->mpDiagnosticsMessage," << endl <<
    "                             strlen(pDevice->mpDiagnosticsMessage)))" << endl <<
    "            {" << endl <<
    "                printk(KERN_INFO \"Error in " << GetGuidelines().FirstClassName() << "_ioctl, copy_from_user routine.\\n\");" << endl <<
    "                up(&gSem);" << endl <<
    "                return -EFAULT;" << endl <<
    "            }" << endl <<
    "            break;" << endl <<
    "            " << endl <<
    "        case CODE_SKETCH_COMMAND_SEEK_FORWARD:" << endl <<
    "" << endl <<
    "            break;" << endl <<
    "            " << endl <<
    "        case CODE_SKETCH_COMMAND_SEEK_BACK:" << endl <<
    "" << endl <<
    "            break;" << endl <<
    "            " << endl <<
    "        default:" << endl <<
    "            // Do nothing" << endl <<
    "            break;" << endl <<
    "            " << endl <<
    "        }" << endl <<
    "    }" << endl <<
    "    up(&gSem);" << endl <<
    "    return 0;" << endl <<
    "}" << endl <<
    "" << endl <<
    "ssize_t" << endl <<
    "" << GetGuidelines().FirstClassName() << "_read(struct file * apFile, char * apBuffer, size_t aLength, loff_t * aOffset)" << endl <<
    "{" << endl <<
    "    " << GetGuidelines().FirstClassName() << "Data * pDevice = 0;" << endl <<
    "    ssize_t bytesRead = 0;" << endl <<
    "    " << endl <<
    "    if( !apFile || !apFile->private_data )" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"ERROR in " << GetGuidelines().FirstClassName() << "_read: NULL input detected.\\n\");" << endl <<
    "        return -EINVAL;" << endl <<
    "    }" << endl <<
    "    pDevice = apFile->private_data;" << endl <<
    "" << endl <<
    "    // This device remembers where its seek position is, even after the" << endl <<
    "    // associated node is opened or closed repeatedly." << endl <<
    "    if( pDevice->mOffset > *aOffset  )" << endl <<
    "    {" << endl <<
    "        *aOffset = pDevice->mOffset;" << endl <<
    "    }" << endl <<
    "    " << endl <<
    "    if( *aOffset >= pDevice->mSize )" << endl <<
    "    {" << endl <<
    "        *aOffset = 0;" << endl <<
    "        pDevice->mOffset = *aOffset;" << endl <<
    "        return 0;" << endl <<
    "    }" << endl <<
    "    " << endl <<
    "    if (down_interruptible(&gSem))" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"Error in " << GetGuidelines().FirstClassName() << "_read, down_interruptible call.\\n\");" << endl <<
    "        return -ERESTARTSYS;" << endl <<
    "    }" << endl <<
    "    " << endl <<
    "    if (copy_to_user(apBuffer, &pDevice->mpBuffer[*aOffset], pDevice->mSize - *aOffset))" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"Error in " << GetGuidelines().FirstClassName() << "_read, copy_from_user routine.\\n\");" << endl <<
    "        up(&gSem);" << endl <<
    "        return -EFAULT;" << endl <<
    "    }" << endl <<
    "    " << endl <<
    "    bytesRead = pDevice->mSize - *aOffset;" << endl <<
    "    *aOffset += bytesRead;" << endl <<
    "    pDevice->mOffset = *aOffset;" << endl <<
    "    " << endl <<
    "    up(&gSem);" << endl <<
    "    return bytesRead;" << endl <<
    "}" << endl <<
    "" << endl <<
    "ssize_t" << endl <<
    "" << GetGuidelines().FirstClassName() << "_write( struct file *     apFile," << endl <<
    "                      const char *         apBuffer," << endl <<
    "                      size_t               aLength," << endl <<
    "                      loff_t *             aOffset)" << endl <<
    "{" << endl <<
    "    " << GetGuidelines().FirstClassName() << "Data * pDevice = 0;" << endl <<
    "" << endl <<
    "" << endl <<
    "    if( !apFile || !apFile->private_data )" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"ERROR in " << GetGuidelines().FirstClassName() << "_write: NULL input detected.\\n\");" << endl <<
    "        return -EINVAL;" << endl <<
    "    }" << endl <<
    "    pDevice = apFile->private_data;" << endl <<
    "" << endl <<
    "    if( aLength >  MAX_FULL_DRIVER_STORAGE - 1)" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"ERROR in " << GetGuidelines().FirstClassName() << "_write: Input is too long to accept.\\n\");" << endl <<
    "        return -ENOMEM;" << endl <<
    "    }" << endl <<
    "    if (down_interruptible(&gSem))" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"Error in " << GetGuidelines().FirstClassName() << "_write, down_interruptible call.\\n\");" << endl <<
    "        return -ERESTARTSYS;" << endl <<
    "    }" << endl <<
    "    " << endl <<
    "    if (copy_from_user(pDevice->mpBuffer, apBuffer, aLength))" << endl <<
    "    {" << endl <<
    "        printk(KERN_INFO \"Error in " << GetGuidelines().FirstClassName() << "_write, copy_from_user routine.\\n\");" << endl <<
    "        up(&gSem);" << endl <<
    "        return -EFAULT;" << endl <<
    "    }" << endl <<
    "    pDevice->mpBuffer[aLength] = '\\0';" << endl <<
    "    pDevice->mSize = aLength;" << endl <<
    "    " << endl <<
    "    *aOffset += aLength;" << endl <<
    "    up(&gSem);" << endl <<
    "    return aLength;" << endl <<
    "" << endl <<
    "}" << endl;
}
///////////////////////////////////////////////////////////////////////////////
// class CommonDeviceDriverRoutinesGenerator

CommonDeviceDriverRoutinesGenerator::CommonDeviceDriverRoutinesGenerator( const Guidelines& arGuidelines )
    :    GeneratorBase( arGuidelines ),
        mFileCreator ("StandardCodeSketchCommands.h",
                      arGuidelines.OutputDirectory(),
                      arGuidelines.FileCreationFlags())
{
}

CommonDeviceDriverRoutinesGenerator::~CommonDeviceDriverRoutinesGenerator()
{
}

void CommonDeviceDriverRoutinesGenerator::CreateProlog()
{
}
void CommonDeviceDriverRoutinesGenerator::CreateBody(const std::string& arClassName)
{
    // Note: it is really better to use the file  StandardCodeSketchCommands.h,
    // which is supplied with the CodeSketch source code.  I generate the
    // contents of that file, verbatim, here, in order to make CodeSketch work
    // "out of the box" (i.e., no modifications required in order to create code
    // that will build the first time).

    SendToFile() <<
    "#ifndef JFH_STANDARD_CODE_SKETCH_COMMANDS_H" << endl <<
    "#define JFH_STANDARD_CODE_SKETCH_COMMANDS_H" << endl <<
    "" << endl <<
    "#define STANDARD_COMMAND_BUFFER_SIZE_CODE_SKETCH  512" << endl <<
    "" << endl <<
    "#ifdef __cplusplus" << endl <<
    "extern \"C\" {" << endl <<
    "#endif" << endl <<
    "" << endl <<
    "typedef struct StandardCodeSketchCommands" << endl <<
    "{" << endl <<
    "     long    mCommand;" << endl <<
    "     long    mDataIn;" << endl <<
    "     long    mDataOut;" << endl <<
    "     char    mBufferIn [ STANDARD_COMMAND_BUFFER_SIZE_CODE_SKETCH ];" << endl <<
    "     char    mBufferOut[ STANDARD_COMMAND_BUFFER_SIZE_CODE_SKETCH ];" << endl <<
    "} StandardCodeSketchCommands;" << endl <<
    "" << endl <<
    "#define CODE_SKETCH_OVERALL_COMMAND_TO_IOCTL         1000" << endl <<
    "#define CODE_SKETCH_COMMAND_DUMP                     1001" << endl <<
    "#define CODE_SKETCH_COMMAND_SEEK_FORWARD             1002" << endl <<
    "#define CODE_SKETCH_COMMAND_SEEK_BACK                1003" << endl <<
    "" << endl <<
    "#ifdef __cplusplus" << endl <<
    " }" << endl <<
    "#endif" << endl <<
    "" << endl <<
    "#endif /* JFH_STANDARD_CODE_SKETCH_COMMANDS_H */" << endl;
}

void CommonDeviceDriverRoutinesGenerator::CreateEpilog()
{
}
///////////////////////////////////////////////////////////////////////////////
// class DeviceDriverBuildScriptGenerator

DeviceDriverBuildScriptGenerator::DeviceDriverBuildScriptGenerator( const Guidelines& arGuidelines )
    :    GeneratorBase( arGuidelines ),
        mFileCreator (arGuidelines.DeviceDriverScriptName(),
                      arGuidelines.OutputDirectory(),
                      arGuidelines.FileCreationFlags())
{
}

DeviceDriverBuildScriptGenerator::~DeviceDriverBuildScriptGenerator()
{
}

void DeviceDriverBuildScriptGenerator::CreateHeader()
{
    SendToFile() <<
    "#!/bin/bash" << endl;

    CreateStandardScriptHeader(mFileCreator.FileName(), "");

    SendToFile() << endl <<
    "#" << endl <<
    "# File Contents: This is a bash script to build real device drivers for Linux." << endl <<
    "# Please note that there are some commands in this script that require bash," << endl <<
    "# not just sh." << endl <<
    "#" << endl <<
    "# Usage:" << endl <<
    "#" << endl <<
    "#   BuildDeviceDrivers.sh  [-tidy] [-verbose]" << endl <<
    "#" << endl <<
    "#      -tidy removes the object files from the driver directory," << endl <<
    "#      so as to make it easier to type the insmod command (for example)." << endl <<
    "#" << endl <<
    "#      -verbose shows more details as to what compile and link commands" << endl <<
    "#      are being issued." << endl <<
    "#" << endl <<
    "# To add an additional, new driver, simply follow these easy steps:" << endl <<
    "#" << endl <<
    "#     1. Add the name of the new driver to the PROGRAMS variable." << endl <<
    "#     2. Create a new variable, whose name is aaa_FILES, where \"aaa\" refers to" << endl <<
    "#        the new name that you added in step 1." << endl <<
    "#     3. Add source files to the variable you created in step 2." << endl <<
    "#     4. Add any common source files to the COMMON_FILES variable." << endl <<
    "#" << endl <<
    "###############################################################################" << endl <<
    "" << endl;
}

void DeviceDriverBuildScriptGenerator::CreateProlog()
{
    SendToFile() << endl <<
    "PROGRAMS=\"" << GetGuidelines().FirstClassName() << "\"" << endl <<
    "" << endl <<
    "# Files for each driver, and files common to more than one driver. Please note" << endl <<
    "# that full or relative paths may be used, as long as the base file names are" << endl <<
    "# unique." << endl <<
    "" << endl <<
    "" << GetGuidelines().FirstClassName() << "_FILES=\"" <<
          GetGuidelines().FirstClassName() << ".c\"" << endl <<
    "COMMON_FILES=" << endl <<
    "" << endl <<
    "LYNXOS_KERNEL_SYMBOLS_IMPORT_FILENAME=" << GetGuidelines().LynxOSImportsFileName() << endl <<
    "source " << GetGuidelines().CommonBuildScriptName() << endl;
}

///////////////////////////////////////////////////////////////////////////////
// class DeviceDriverBuildScriptGenerator

CommonBuildScriptGenerator::CommonBuildScriptGenerator( const Guidelines& arGuidelines )
    :    GeneratorBase( arGuidelines ),
        mFileCreator (arGuidelines.CommonBuildScriptName(),
                      arGuidelines.OutputDirectory(),
                      arGuidelines.FileCreationFlags())
{
}

CommonBuildScriptGenerator::~CommonBuildScriptGenerator()
{
}

void CommonBuildScriptGenerator::CreateHeader()
{
    SendToFile() <<
    "#!/bin/bash" << endl;

    CreateStandardScriptHeader(mFileCreator.FileName(), "");

    SendToFile() << endl <<
    "# Usage: Source this file from within your specific build script." << endl <<
    "#" << endl <<
    "#      -tidy removes the object files from the driver directory," << endl <<
    "#      so as to make it easier to type the insmod command (for example)." << endl <<
    "#" << endl <<
    "#      -verbose shows more details as to what compile and link commands" << endl <<
    "#      are being issued." << endl <<
    "#" << endl <<
    "###############################################################################" << endl <<
    "" << endl;
}

void CommonBuildScriptGenerator::CreateProlog()
{
    SendToFile() <<
    "echo \"Performing a clean build of the following drivers: ${PROGRAMS}\"" << endl <<
    "" << endl <<
    "if [ ${TARGET} = \"LINUX_X86\" ];then" << endl <<
    "    CC=gcc" << endl <<
    "    LINK=\"ld -r\"" << endl <<
    "    KERNELDIR=\"/usr/src/linux-2.4\"" << endl <<
    "    INCLUDES=\"-I ${KERNELDIR}/include -I . \"" << endl <<
    "    " << endl <<
    "    CFLAGS=\"-D __KERNEL__  -D MODULE -O2 -Wall -Wno-unknown-pragmas -Wno-unused\"" << endl <<
    "    " << endl <<
    "elif [ ${TARGET} = \"LYNXOS_PPC\" ];then" << endl <<
    "    CC=gcc" << endl <<
    "    LINK=\"ld -bM:SRE -bimport:${LYNXOS_KERNEL_SYMBOLS_IMPORT_FILENAME}\"" << endl <<
    "    INCLUDES=\"\"" << endl <<
    "    INCLUDES=\"${INCLUDES} -I/usr/lynx/3.1.0/ppc/sys/include/kernel\"" << endl <<
    "    INCLUDES=\"${INCLUDES} -I/usr/lynx/3.1.0/ppc/sys/include/family/ppc\"" << endl <<
    "    INCLUDES=\"${INCLUDES} -I/usr/lynx/3.1.0/ppc/sys/lynx.os\"" << endl <<
    "    " << endl <<
    "    CFLAGS=\"-D${TARGET} -O2 -Wall -Wno-unknown-pragmas -Wno-unused -D__NO_INCLUDE_WARN__ -D__LYNXOS\"" << endl <<
    "else" << endl <<
    "    echo \"ERROR: TARGET=${TARGET}, but only LINUX_X86 and LYNXOS_PPC are supported.\"" << endl <<
    "    exit -1       " << endl <<
    "fi" << endl <<
    "    " << endl <<
    "if [ -d ${TARGET} ];then rm -r ${TARGET}; fi" << endl <<
    "if [ ! -d ${TARGET} ];then mkdir ${TARGET}; fi" << endl <<
    "" << endl <<
    "# Search input parameters for position-independent switches:" << endl <<
    "VERBOSE=\"false\"" << endl <<
    "TIDY=\"false\"" << endl <<
    "" << endl <<
    "for ARGS in $*" << endl <<
    "do" << endl <<
    "    if [ ${ARGS} = \"-tidy\" ] ;  then TIDY=\"true\";    fi" << endl <<
    "    if [ ${ARGS} = \"-verbose\" ];then VERBOSE=\"true\"; fi" << endl <<
    "done" << endl <<
    "" << endl <<
    "function compile_files" << endl <<
    "{" << endl <<
    "    local sourceFile" << endl <<
    "    for sourceFile in $*" << endl <<
    "    do" << endl <<
    "        echo \"Compiling ${sourceFile} ...\"" << endl <<
    "        MakeObjectFileFromSourceFile ${sourceFile}" << endl <<
    "        " << endl <<
    "        if [ ${VERBOSE} = \"true\" ]; then" << endl <<
    "            echo \"${CC} -c ${CFLAGS} ${INCLUDES} ${sourceFile} -o ${OBJ_FILE}\"" << endl <<
    "        fi" << endl <<
    "        " << endl <<
    "        ${CC} -c ${CFLAGS} ${INCLUDES} ${sourceFile} -o ${OBJ_FILE}" << endl <<
    "    done" << endl <<
    "}" << endl <<
    "" << endl <<
    "function MakeObjectFileFromSourceFile" << endl <<
    "{" << endl <<
    "    local sourceFile=$@" << endl <<
    "    # Strip off any reference to another directory (such as ../otherDir/)" << endl <<
    "    # This has the effect of requiring unique source file names, even" << endl <<
    "    # though they may reside in different directories." << endl <<
    "    sourceFile=${sourceFile##*/}" << endl <<
    "    objFile=${sourceFile%.*}.o" << endl <<
    "    OBJ_FILE=\"${TARGET}/${objFile}\"" << endl <<
    "}" << endl <<
    "" << endl <<
    "function MakeObjectFileListFromSourceFileList" << endl <<
    "{" << endl <<
    "    # OBJ_FILE_LIST is not local; function result is placed in this variable." << endl <<
    "    OBJ_FILE_LIST=\"\"" << endl <<
    "    for sourceFile in $*" << endl <<
    "    do" << endl <<
    "        MakeObjectFileFromSourceFile ${sourceFile}" << endl <<
    "        OBJ_FILE_LIST=\"${OBJ_FILE_LIST} ${OBJ_FILE}\"" << endl <<
    "    done" << endl <<
    "}" << endl <<
    "" << endl <<
    "function MakeSourceFileListFromProgramName" << endl <<
    "{" << endl <<
    "    # SOURCE_FILE_LIST is not local; function result is placed in this variable." << endl <<
    "    SOURCE_FILE_LIST=\"\"" << endl <<
    "    local programName=$1" << endl <<
    "    local temp=\"${programName}_FILES\"" << endl <<
    "    eval SOURCE_FILE_LIST=$(echo \"\\${${temp}}\")" << endl <<
    "}" << endl <<
    "" << endl <<
    "function link_files" << endl <<
    "{" << endl <<
    "    local programName=$1" << endl <<
    "    local outputFile=${TARGET}/${programName}.drv" << endl <<
    "    echo \"Linking ${outputFile}...\"" << endl <<
    "    MakeSourceFileListFromProgramName ${programName}" << endl <<
    "    " << endl <<
    "    MakeObjectFileListFromSourceFileList ${SOURCE_FILE_LIST} ${COMMON_FILES}" << endl <<
    "    " << endl <<
    "    if [ ${VERBOSE} = \"true\" ]; then" << endl <<
    "        echo \"${LINK} ${OBJ_FILE_LIST} -o ${outputFile}\"" << endl <<
    "    fi" << endl <<
    "    ${LINK} ${OBJ_FILE_LIST} -o ${outputFile}" << endl <<
    "}" << endl <<
    "" << endl <<
    "# Compile files that are unique to each program:" << endl <<
    "for programName in ${PROGRAMS}" << endl <<
    "do" << endl <<
    "    MakeSourceFileListFromProgramName ${programName}" << endl <<
    "    compile_files ${SOURCE_FILE_LIST}" << endl <<
    "done" << endl <<
    "" << endl <<
    "# Compile files that shared among two or more programs:" << endl <<
    "compile_files ${COMMON_FILES}" << endl <<
    "" << endl <<
    "# Link each program:" << endl <<
    "for name in ${PROGRAMS}" << endl <<
    "do" << endl <<
    "    link_files ${name}" << endl <<
    "done" << endl <<
    "" << endl <<
    "if [ ${TIDY} = \"true\" ]; then" << endl <<
    "    echo \"Removing intermediate files, because you specified -tidy...\"" << endl <<
    "    rm ${TARGET}/*.o" << endl <<
    "fi        " << endl;
}

} // end of the code_sketch namespace
