/* ****************************************************************************
 *  FileCreator.cpp
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.6 $
 *  $Date: 2002/12/14 20:52:17 $
 *  $Author: jhubbard $
 *************************************************************************** */
#include <sys/types.h> // for stat
#include <sys/stat.h>  // for stat

#ifndef WIN32
	#include <unistd.h>    // for stat
	#include <errno.h>
#endif

#include <string.h>
#include <string>
#include <iostream>
#include <fstream>
#include "FileCreator.h"

namespace code_sketch
{
FileCreator::FileCreator(const std::string& arFile,
						 const std::string& arOutputDir,
						 int aMode)
	:	mFileName( arFile )
{
	// Please note that this code may look more complex than it needs to be, but
	// it is that way in order to support the older, non-standard C++ library that
	// ships with gcc/g++ 2.95.3.  For example, gcc 2.95.3 cannot throw an
	// ios::failure, etc, etc.

	bool bOverwriteRequested = ((aMode & OVERWRITE) != 0);
	struct stat fileInfo;
	std::string fullPath( arOutputDir );
	if( fullPath.length() > 0 && fullPath.substr(fullPath.length() - 1, 1) != "/" )
	{
		fullPath += '/';
	}
	fullPath += arFile;
	
	int result = stat(fullPath.c_str(), &fileInfo);
	bool bFileAlreadyExists = ( 0 == result);
	
	if( bFileAlreadyExists && (! bOverwriteRequested) )
	{
		std::cout << "Did NOT create " << fullPath
				  << " because it already exists." << std::endl;
	}
	else if (bOverwriteRequested || ! bFileAlreadyExists )
	{
		mFile.open(fullPath.c_str(), std::ios::out);

		if (mFile.good() && (aMode & VERBOSE))
		{
			if( bOverwriteRequested && bFileAlreadyExists )
			{
				std::cout << "By your request, overwrote " << fullPath
					      << " with a new file." << std::endl;
			}
			else
			{
				std::cout << "Created " << fullPath << std::endl;
			}
		}
		else if( !mFile.good() )
		{
			std::string strMessage =  std::string("Failed to open ") +
							fullPath + ". Reason: " +
							std::string(strerror(errno)) + "\n";
			throw (strMessage);
		}
	}
}

FileCreator::~FileCreator()
{
	if( mFile )
	{	mFile.close();
	}
}

} // end of the code_sketch namespace
