/* ****************************************************************************
 *  FileCreator.h
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.6 $
 *  $Date: 2002/12/14 20:52:17 $
 *  $Author: jhubbard $
 *************************************************************************** */
#ifndef JFH_CODE_SKETCH_FILECREATOR_H
#define JFH_CODE_SKETCH_FILECREATOR_H

#include <fstream>
#include <string>

namespace code_sketch
{

/** Purpose: Handle file input/output, including opening, closing, and
	sending characters to the file.  The file is opened upon construction
	of a new FileCreator object, and closed with the destruction of the
	object.  In between, text may be written to the file via the SendToFile()
	method.
 */
class FileCreator
{
public:
    /// Constructor.
    FileCreator(const std::string& arFile, const std::string& arOutputDir, int aMode);
    virtual ~FileCreator();
	std::ostream& SendToFile() { return mFile; };
	
	/** Returns the file name that was passed to the constructor.  You can probably
		do this with the ofstream object mFile, and sooner or later I'll look up
		just exactly how to extract that information.
	*/
	std::string FileName() const { return mFileName; };

private:
	// Copying or assignment of this class is prohibited:
    FileCreator(const FileCreator&);
    FileCreator& operator=(const FileCreator&);
	
	std::ofstream mFile;
	std::string   mFileName;
};

const static int VERBOSE = 1;
const static int OVERWRITE = 2;

} // end of the code_sketch namespace

#endif // #ifndef JFH_CODE_SKETCH_FILECREATOR_H

