/* ****************************************************************************
 *  Guidelines.h
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.10 $
 *  $Date: 2002/12/15 05:03:19 $
 *  $Author: jhubbard $
 *************************************************************************** */
#ifndef JFH_CODE_SKETCH_GUIDELINES_H
#define JFH_CODE_SKETCH_GUIDELINES_H

#include "CommandLine.h"
#include <string>
#include <vector>

namespace code_sketch
{

/** Purpose: TODO: Describe the purpose of the class.
 *  (Everything that you write here will show up in the
 *  doc++ generated documentation.)
 */
class Guidelines : public tools::CommandLine
{
public:
    /// Constructor.
    Guidelines(int argc, char* argv[]);
    virtual ~Guidelines();

	std::string 	FirstClassName()    		const;
	std::vector<std::string>&  ClassList()      const;
	std::string 	ClassHeaderName() 	 		const;
	std::string 	ClassBodyName()  	 		const;
	std::string 	ClassInlineName()   		const;
	std::string 	MakefileName()	 		const;
	std::string 	DeviceDriverScriptName() 	const;
	std::string 	CommonBuildScriptName() 	const;
	std::string 	UnitTestName() 				const;
	std::string		BaseUnitTestName() 		   	const;

	std::string 	LynxOSInfoMainFileName() 	const;
	std::string 	LynxOSInfoHeaderFileName() 	const;
	std::string 	LynxOSImportsFileName() 	const;

	std::string 	Namespace()	 				const;
	std::string 	Author()	     			const;
	std::string		Copyright() 	  			const;
	std::string		BaseFilename() 				const;
	std::string 	TargetOperatingSystem() 	const;
	std::string 	OutputDirectory() 			const;
	std::string 	CustomInitials() 			const;

	int				FileCreationFlags() 	  	const;

	bool			ModeIsCplusplus()			const;
	bool			ModeIsAnsiC()				const;
	bool			ModeIsDeviceDriver()	   	const;
	bool 			IsCharDriver() 				const;
	bool 			IsBlockDriver() 			const;
	bool 			SparseCodeGenerationDesired() const;
	
	bool			BuildScriptDesired()	  	const;
	bool			UnitTestDesired()  			const;
	bool			InlineFileDesired()  		const;
	bool			DumpDiagnosticsDesired()	const;
	bool			NamespaceDesired()			const;
	bool			OpenSourceDesired()   		const;
	bool			SccsKeywordsDesired()     	const;
	bool			PureInterfaceClassDesired() const;
	bool 			HelpRequested() 			const;
	bool 			VersionReportRequested() 	const;
	bool 			DiagnosticsReportRequested() const;

	void DumpDiagnostics();

private:
	// Copying or assignment of this class is prohibited:
    Guidelines(const Guidelines&);
    Guidelines& operator=(const Guidelines&);

	// Cache the parsed list of classes:
	mutable std::vector<std::string>	mClassList;
	std::string					mFirstClassName;

	void ParseClassList();
};
} // end of the code_sketch namespace

#endif // #ifndef JFH_CODE_SKETCH_GUIDELINES_H

