/* ****************************************************************************
 *  LynxOSDeviceDriverGeneration.cpp
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.5 $
 *  $Date: 2002/12/16 21:00:25 $
 *  $Author: jhubbard $
 *************************************************************************** */
#include <string>
#include <iostream>
#include "DeviceDriverGeneration.h"
#include "Guidelines.h"

namespace code_sketch
{
using std::endl;
///////////////////////////////////////////////////////////////////////////////
// class LynxOS_ppc_DeviceDriverGenerator

LynxOS_ppc_DeviceDriverGenerator::LynxOS_ppc_DeviceDriverGenerator( const Guidelines& arGuidelines )
    :    GeneratorBase( arGuidelines ),
        mFileCreator (arGuidelines.ClassBodyName(),
                      arGuidelines.OutputDirectory(),
                      arGuidelines.FileCreationFlags())
{
}

LynxOS_ppc_DeviceDriverGenerator::~LynxOS_ppc_DeviceDriverGenerator()
{
}

void LynxOS_ppc_DeviceDriverGenerator::CreateBody(const std::string& arClassName)
{
    SendToFile() <<
    "#include <conf.h>" << endl <<
    "#include <kern_proto.h>" << endl <<
    "#include <kernel.h>" << endl <<
    "#include <sys/file.h>" << endl <<
    "#include <file.h>" << endl <<
    "#include <dldd.h>" << endl <<
    "#include <string.h>" << endl <<
    "#include \"" << GetGuidelines().LynxOSInfoHeaderFileName() << "\"" << endl <<
    "" << endl <<
    "#if (!defined LYNXOS_PPC) && (!defined LYNXOS_X86)" << endl <<
    "    #error \"This is a LynxOS driver, but neither LYNXOS_PPC nor LYNXOS_X86 is defined\"" << endl <<
    "#endif" << endl <<
    "" << endl <<
    "void " << GetGuidelines().FirstClassName() << "DriverDiagnosticsDump_Statics(const " << GetGuidelines().FirstClassName() << "DriverStatics * apStatics);" << endl <<
    "" << endl <<
    "int" << endl <<
    "" << GetGuidelines().FirstClassName() << "_Open(" << GetGuidelines().FirstClassName() << "DriverStatics * apStatics, int aDeviceNo, struct file * f)" << endl <<
    "{" << endl <<
    "    kkprintf(\"Entering " << GetGuidelines().FirstClassName() << "_Open. Device number: %d.\\n\", aDeviceNo);" << endl <<
    "    return 0;" << endl <<
    "}" << endl <<
    "" << endl <<
    "int" << endl <<
    "" << GetGuidelines().FirstClassName() << "_Close(" << GetGuidelines().FirstClassName() << "DriverStatics * apStatics, struct file * f)" << endl <<
    "{" << endl <<
    "    kkprintf(\"Entering " << GetGuidelines().FirstClassName() << "_Close...\\n\");" << endl <<
    "    return 0;" << endl <<
    "}" << endl <<
    "" << endl <<
    "int" << endl <<
    "" << GetGuidelines().FirstClassName() << "_Read(" << GetGuidelines().FirstClassName() << "DriverStatics *     apStatics," << endl <<
    "          struct file *         f," << endl <<
    "          char *                 buff," << endl <<
    "          register int             count)" << endl <<
    "{" << endl <<
    "    kkprintf(\"Entering " << GetGuidelines().FirstClassName() << "_Read...\\n\");" << endl <<
    "    " << GetGuidelines().FirstClassName() << "DriverDiagnosticsDump_Statics( apStatics );" << endl <<
    "    return 0;" << endl <<
    "}" << endl <<
    "" << endl <<
    "int" << endl <<
    "" << GetGuidelines().FirstClassName() << "_Write(" << GetGuidelines().FirstClassName() << "DriverStatics *     apStatics," << endl <<
    "           struct file *         f," << endl <<
    "           char *                 buff," << endl <<
    "           register int         count)" << endl <<
    "{" << endl <<
    "    kkprintf(\"Entering " << GetGuidelines().FirstClassName() << "_Write...\\n\");" << endl <<
    "    " << GetGuidelines().FirstClassName() << "DriverDiagnosticsDump_Statics( apStatics );" << endl <<
    "    return count;" << endl <<
    "}" << endl <<
    "" << endl <<
    "int " << GetGuidelines().FirstClassName() << "_Select()" << endl <<
    "{" << endl <<
    "    return SYSERR;" << endl <<
    "}" << endl <<
    "" << endl <<
    "int" << endl <<
    "" << GetGuidelines().FirstClassName() << "_Ioctl(" << GetGuidelines().FirstClassName() << "DriverStatics * apStatics," << endl <<
    "           struct file * f," << endl <<
    "           int aCommand," << endl <<
    "           char * apArg)" << endl <<
    "{" << endl <<
    "    kkprintf(\"Entering " << GetGuidelines().FirstClassName() << "_Ioctl, with command number ...\\n\", aCommand);" << endl <<
    "    " << endl <<
    "    return 0;" << endl <<
    "}" << endl <<
    "" << endl <<
    "char *" << endl <<
    "" << GetGuidelines().FirstClassName() << "_Install(" << GetGuidelines().FirstClassName() << "DriverInfo * apInfo)" << endl <<
    "{" << endl <<
    "    " << GetGuidelines().FirstClassName() << "DriverStatics * pStatics = 0;" << endl <<
    "    kkprintf(\"Entering " << GetGuidelines().FirstClassName() << "_Install...\");" << endl <<
    "" << endl <<
    "    if( apInfo )" << endl <<
    "    {" << endl <<
    "        pStatics = (" << GetGuidelines().FirstClassName() << "DriverStatics *)sysbrk( (long)sizeof(*pStatics) );" << endl <<
    "        if( pStatics )" << endl <<
    "        {" << endl <<
    "            memset(pStatics, 0, sizeof(*pStatics));" << endl <<
    "            pStatics->mVendorID = apInfo->mVendorID;" << endl <<
    "            pStatics->mDeviceID = apInfo->mDeviceID;" << endl <<
    "            pStatics->mYourCustomDataHere = 550; // For example..." << endl <<
    "" << endl <<
    "            " << GetGuidelines().FirstClassName() << "DriverDiagnosticsDump_Statics( pStatics );" << endl <<
    "        }" << endl <<
    "    }" << endl <<
    "    else" << endl <<
    "    {" << endl <<
    "        kkprintf(\"Null apInfo pointer was passed in.\\n\");" << endl <<
    "        return (char *) SYSERR;" << endl <<
    "    }" << endl <<
    "" << endl <<
    "    return (char *) pStatics;" << endl <<
    "}" << endl <<
    "" << endl <<
    "int" << endl <<
    "" << GetGuidelines().FirstClassName() << "_Uninstall(" << GetGuidelines().FirstClassName() << "DriverStatics * apStatics)" << endl <<
    "{" << endl <<
    "    kkprintf(\"Entering " << GetGuidelines().FirstClassName() << "_Uninstall...\\n\");" << endl <<
    "    " << GetGuidelines().FirstClassName() << "DriverDiagnosticsDump_Statics( apStatics );" << endl <<
    "" << endl <<
    "    sysfree((void*)apStatics, (long)sizeof(*apStatics));" << endl <<
    "    return 0;" << endl <<
    "}" << endl <<
    "" << endl <<
    "#ifdef LYNXOS_X86" << endl <<
    "    static" << endl <<
    "#endif // LYNXOS_X86" << endl <<
    "" << endl <<
    "    struct dldd entry_points =" << endl <<
    "    {" << endl <<
    "        " << GetGuidelines().FirstClassName() << "_Open," << endl <<
    "        " << GetGuidelines().FirstClassName() << "_Close," << endl <<
    "        " << GetGuidelines().FirstClassName() << "_Read," << endl <<
    "        " << GetGuidelines().FirstClassName() << "_Write," << endl <<
    "        " << GetGuidelines().FirstClassName() << "_Select," << endl <<
    "        " << GetGuidelines().FirstClassName() << "_Ioctl," << endl <<
    "        " << GetGuidelines().FirstClassName() << "_Install," << endl <<
    "        " << GetGuidelines().FirstClassName() << "_Uninstall," << endl <<
    "        (char *) 0" << endl <<
    "    };" << endl <<
    "" << endl <<
    "void" << endl <<
    "" << GetGuidelines().FirstClassName() << "DriverDiagnosticsDump_Statics(const "
	   << GetGuidelines().FirstClassName() << "DriverStatics * apStatics)" << endl <<
    "{" << endl <<
    "    if( apStatics )" << endl <<
    "    {" << endl <<
    "        kkprintf(\"" << GetGuidelines().FirstClassName() << "DriverDiagnosticsDump_Statics. \"" << endl <<
    "                 \"\\n\\tmVendorID: %x.\"" << endl <<
    "                 \"\\n\\tmDeviceID: %x\"" << endl <<
    "                 \"\\n\\tYourCustomDataHere: %x\\n\"," << endl <<
    "                 apStatics->mVendorID," << endl <<
    "                 apStatics->mDeviceID," << endl <<
    "                 apStatics->mYourCustomDataHere);" << endl <<
    "    }" << endl <<
    "    else" << endl <<
    "    {" << endl <<
    "        kkprintf(\"" << GetGuidelines().FirstClassName() << "DriverDiagnosticsDump_Statics: Null apStatics pointer was passed in.\\n\");" << endl <<
    "    }" << endl <<
    "}" << endl <<
    "" << endl;
}

///////////////////////////////////////////////////////////////////////////////
// class LynxOS_InfoMainGenerator

LynxOS_InfoMainGenerator::LynxOS_InfoMainGenerator( const Guidelines& arGuidelines )
    :    GeneratorBase( arGuidelines ),
        mFileCreator (arGuidelines.LynxOSInfoMainFileName(),
                      arGuidelines.OutputDirectory(),
                      arGuidelines.FileCreationFlags())
{
}

LynxOS_InfoMainGenerator::~LynxOS_InfoMainGenerator()
{
}
void LynxOS_InfoMainGenerator::CreateBody(const std::string& arClassName)
{
    SendToFile() <<
    "#include <unistd.h>" << endl <<
    "#include <stdio.h>" << endl <<
    "#include \"" << GetGuidelines().FirstClassName() << "_info.h\"" << endl <<
    "" << endl <<
    "int main()" << endl <<
    "{" << endl <<
    "    long YOUR_VENDOR        = 1;" << endl <<
    "    long YOUR_DEVICE_ID     = 2;" << endl <<
    "" << endl <<
    "    " << GetGuidelines().FirstClassName() << "DriverInfo  driverInfo =" << endl <<
    "    {" << endl <<
    "        YOUR_VENDOR," << endl <<
    "        YOUR_DEVICE_ID" << endl <<
    "    };" << endl <<
    "" << endl <<
    "    write(1, &driverInfo, sizeof(driverInfo));" << endl <<
    "    return 0;" << endl <<
    "}" << endl;
}

///////////////////////////////////////////////////////////////////////////////
// class LynxOS_InfoHeaderGenerator

LynxOS_InfoHeaderGenerator::LynxOS_InfoHeaderGenerator( const Guidelines& arGuidelines )
    :    GeneratorBase( arGuidelines ),
        mFileCreator (arGuidelines.LynxOSInfoHeaderFileName(),
                      arGuidelines.OutputDirectory(),
                      arGuidelines.FileCreationFlags())
{
}

LynxOS_InfoHeaderGenerator::~LynxOS_InfoHeaderGenerator()
{
}

void LynxOS_InfoHeaderGenerator::CreateBody(const std::string& arClassName)
{
    SendToFile() <<
    "#ifndef " << MangledUpperCaseName( GetGuidelines() )     << endl <<
    "#define " << MangledUpperCaseName( GetGuidelines() )     << endl <<
    ""                                                         << endl <<
    "#ifdef __cplusplus"                                     << endl <<
    "extern \"C\""                                             << endl <<
    "{"                                                        << endl <<
    "#endif //  __cplusplus"                                 << endl <<
    ""                                                         << endl <<
    "/** This info structure is required for a LynxOS driver.  The contents may be anything," << endl <<
    "    but a non-empty structure must exist.  Here, the contents are used as a" << endl <<
    "    way to pass in initialization parameters to the driver.  These will be used" << endl <<
    "    during driver and device (these two terms have specific meanings in the" << endl <<
    "    LynxOS Device Writers Manual) installation." << endl <<
    "*/" << endl <<
    "" << endl <<
    "typedef struct " << GetGuidelines().FirstClassName() << "DriverInfo" << endl <<
    "{" << endl <<
    "    long    mVendorID;" << endl <<
    "    long    mDeviceID;" << endl <<
    "    // ... and feel free to add or change as you like" << endl <<
    "} " << GetGuidelines().FirstClassName() << "DriverInfo;" << endl <<
    "" << endl <<
    "/** The statics structure is passed to the entry points of your device driver." << endl <<
    "    This makes it easy to avoid the use of global variables; easier, in fact," << endl <<
    "    than with Linux, for example. With LynxOS, you can put everything in the" << endl <<
    "    user-defined statics structure." << endl <<
    "*/" << endl <<
    "typedef struct tag" << GetGuidelines().FirstClassName() << "DriverStatics" << endl <<
    "{" << endl <<
    "     long    mVendorID;" << endl <<
    "     long    mDeviceID;" << endl <<
    "     long    mYourCustomDataHere;" << endl <<
    "     // ... and feel free to add or change as you like" << endl <<
    "} " << GetGuidelines().FirstClassName() << "DriverStatics;" << endl <<
    "" << endl <<
    "#ifdef __cplusplus"                                                 << endl <<
    "}"                                                                    << endl <<
    "#endif //  __cplusplus"                                             << endl <<
    ""                                                                    << endl <<
    "#endif // " << MangledUpperCaseName( GetGuidelines() )             << endl <<
    ""                                                                     << endl;
}

///////////////////////////////////////////////////////////////////////////////
// class LynxOS_ppc_ImportsGenerator

LynxOS_ppc_ImportsGenerator::LynxOS_ppc_ImportsGenerator( const Guidelines& arGuidelines )
    :    GeneratorBase( arGuidelines ),
        mFileCreator (arGuidelines.LynxOSImportsFileName(),
                      arGuidelines.OutputDirectory(),
                      arGuidelines.FileCreationFlags())
{
}

LynxOS_ppc_ImportsGenerator::~LynxOS_ppc_ImportsGenerator()
{
}

void LynxOS_ppc_ImportsGenerator::CreateHeader()
{
}

void LynxOS_ppc_ImportsGenerator::CreateProlog()
{
}

void LynxOS_ppc_ImportsGenerator::CreateBody(const std::string& arClassName)
{
    SendToFile() <<
    "sysbrk" << endl <<
    "sysfree" << endl <<
    "memset" << endl <<
    "memcpy" << endl <<
    "kkprintf" << endl <<
    "strcpy" << endl <<
    "recoset" << endl <<
    "noreco" << endl <<
    "byte_swap" << endl <<
    "__outl" << endl <<
    "__inl" << endl <<
    "rbounds" << endl <<
    "wbounds" << endl <<
    "slevel" << endl <<
    "ast2" << endl <<
    "_mail" << endl;
}

void LynxOS_ppc_ImportsGenerator::CreateEpilog()
{
}

} // end of the code_sketch namespace
