#include <rmi/String.h>

String
String::va_format(cchar* format, va_list ap) throw(std::bad_alloc)
{
   // Output of the function is a string built according to the specified
   // format and supplied arguments (as vprintf() does). It uses a static
   // buffer that is immediately copied to a String instance.

   // The buffer is allocated in chunks of 32 bytes. The number 32 is arbitrary.
   // Maximize it to reduce the number of allocations.

   static Lock lock;

   lock.acquire();

   try
   {
      static int    increment = 32;
      static int     cur_size = increment;
      static char* cur_string = new char[cur_size];
      int       required_size = vsnprintf(cur_string, cur_size, format, ap);

      if (cur_size < required_size)
      {
         delete[] cur_string;

         cur_size   = (required_size / increment + 1) * increment;
         cur_string = new char[cur_size];

         vsnprintf(cur_string, cur_size, format, ap);
      }
      String result(cur_string);

      lock.release(); 

      return result;
   }
   catch (...) // Just in case. Don't expect anything except bad_alloc.
   {
      lock.release(); 

      throw std::bad_alloc();
   }
}

String 
String::format(cchar* fmt, ...) throw(std::bad_alloc)
{
   va_list ap;
   return (va_start(ap, fmt), va_format(fmt, ap));
}

Strings
String::split(const String& delimiters, const bool ignore_empty_tokens) const
{
   Strings array;
   
   size_type beg_pos = 0;
   size_type end_pos = 0;
   int num_empty_tokens = 0;
   int i = 0;

   if (size() <= 0)
      return(array);

   //  find all empty tokens on the beginning of the string
   if (!ignore_empty_tokens)
   {
      end_pos = find_first_not_of(delimiters, 0);
      if (end_pos == String::npos)
          end_pos = length() + 1;

      for (i = 0; i < end_pos; i++) 
          array.push_back("");
   }

   while ((beg_pos = find_first_not_of(delimiters, end_pos)) != String::npos) 
   {
      if (!ignore_empty_tokens && ((beg_pos - end_pos) > 1))
      {
          num_empty_tokens = beg_pos - end_pos - 1;
          for (i = 0; i < num_empty_tokens; i++)
             array.push_back("");
      }

      end_pos = find_first_of(delimiters, beg_pos);
   
      if (end_pos == String::npos) end_pos = length();
      
      array.push_back(substr(beg_pos, end_pos - beg_pos));
   }
   // find the empty tokens
   if (!ignore_empty_tokens && end_pos != String::npos)
   {
       num_empty_tokens = length() - end_pos;
       for (i = 0; i < num_empty_tokens; i++)
          array.push_back("");
   }
      
   return array;
}

Strings
String::split(cchar* string, const String& delimiters, const bool ignore_empty_tokens)
{
   // Should be the other way around. 
   // split(delimiters) should call this function for performance purposes.

   return String(string).split(delimiters, ignore_empty_tokens);
}
