#include <rmi/String.h>
#include <rmi/pipe.h>
#include <rmi/connect.h>

using std::string;
using std::vector;

static IO::Pipe  read_;
static IO::Pipe write_;

void
RMI::connect(cchar* read_pipe, cchar* write_pipe)
{
   read_ .open(read_pipe,  IO::Access::read());
   write_.open(write_pipe, IO::Access::write());
}

void
RMI::write(const string& packed)
{
   write_.write(packed.c_str(), packed.size() + 1);
}

string
RMI::read()
{
   char buf[128] = {0};
   int num_bytes = read_.read(buf, 127);

   if (num_bytes == 0) exit(0);

   return buf;
}

// Packed data format is not important and can be proprietary, proprietary
// XML, SOAP, etc. as long as RMI client and server understand each other.
// That simply means that pack/unpack need to be coded appropriately as they
// are used by the client and the server. IMHO the overhead of something like
// SOAP is not justified here as transferred data are well structured and data
// variability is minimal.

static String separator_ = "^";

string
RMI::pack(
   const Parameter& instance, 
   cchar*           function, 
   const Parameter&     arg1, 
   const Parameter&     arg2, 
   const Parameter&     arg3)
{
   typedef const string& strref;

   String packed = instance.string() + separator_ + function;
   
   if (&arg1 != &Parameter::none()) packed += separator_ + strref(arg1);
   if (&arg2 != &Parameter::none()) packed += separator_ + strref(arg2);
   if (&arg3 != &Parameter::none()) packed += separator_ + strref(arg3);

   return packed;
}

RMI::Message
RMI::Message::unpack(const string& packed)
{
   Strings         params = String(packed).split(separator_);
   const string& instance = params[0];
   const string& function = params[1];

   Message msg(instance, function);   

   for (int k = 2; k < params.size(); ++k) msg.args_.push_back(string(params[k]));
   
   return msg;
}

string
RMI::Message::pack() const
{
   return RMI::pack(instance_, function_.c_str(), 
                    0 < args_.size() ? args_[0] : Parameter::none(),
                    1 < args_.size() ? args_[1] : Parameter::none(),
                    2 < args_.size() ? args_[2] : Parameter::none());
}

RMI::Message
RMI::unpack(const string& packed)
{
   return RMI::Message::unpack(packed);
}
