#include "fsm.h"
#include <iostream>
#include <cctype>
#include <cassert>
using namespace std;

class Keyboard : public Fsm {
public:
   Keyboard() : Fsm((State)&Keyboard::initial) {}
   void initial(Event const *e);
   void dfault(Event const *e); // default is a keyword
   void shifted(Event const *e);
};

struct KeyboardEvt : public Event {
   char code;
};

enum {
   SHIFT_DEPRESSED_SIG,
   SHIFT_RELEASED_SIG,
   ANY_KEY_SIG,
};

/*-----------------------------------------------------------------*/
void Keyboard::initial(Event const *e) {
   /* ... initalization of Keyboard attributes */
   cout << "Keyboard initialized";
   tran((State)&Keyboard::dfault);
}

void Keyboard::dfault(Event const *e) {
   switch (e->sig) {
   case SHIFT_DEPRESSED_SIG:
      cout << "default::SHIFT_DEPRESSED";
      tran((State)&Keyboard::shifted);
      break;
   case ANY_KEY_SIG:
      cout << "key " << (char)tolower(((KeyboardEvt *)e)->code);
      break;
   }
}

void Keyboard::shifted(Event const *e) {
   switch (e->sig) {
   case SHIFT_RELEASED_SIG:
      printf("shifted::SHIFT_RELEASED");
      tran((State)&Keyboard::dfault);
      break;
   case ANY_KEY_SIG:
      cout << "key " << (char)toupper(((KeyboardEvt *)e)->code);
      break;
   }
}

int main() {
   Keyboard k;
   k.init();
   for (;;) {
      KeyboardEvt ke;
      cout << "\nSignal<-";
      cin.get(ke.code);        // wait for user input
      cin.get();               // discard '\n'
      switch (ke.code) {
      case '^': ke.sig = SHIFT_DEPRESSED_SIG; break;
      case '6': ke.sig = SHIFT_RELEASED_SIG; break;
      case '.': return 0;
      default: ke.sig = ANY_KEY_SIG; break; 
      }
      k.dispatch(&ke);            // dispatch
   }
   return 0;
}

