#ifndef RMI_CONNECT_H
#define RMI_CONNECT_H

#include <string>
#include <vector>
#include <rmi/parameter.h>

namespace RMI {

typedef const char cchar;

class Message
{
   public:

   const Parameter& instance() const;
   
   const Parameter& arg(int k) const; 
   const std::string& function() const;

   // Convenience function replacing already existing function arguments
   // with a new one. Prepares the message to be sent back to the client
   // with the result as the first and only argument.

   Message& result(const Parameter&);

   std::string pack() const;
   static Message unpack(const std::string&);
   
   private:
   
   Message(const Parameter&, const std::string&);

   Parameter          instance_;
   std::string        function_;
   std::vector<Parameter> args_;
};

inline
const Parameter& 
Message::instance() const 
{ 
   return instance_; 
}

inline
const Parameter& 
Message::arg(int k) const 
{
   return k < args_.size() ? args_[k] : Parameter::none();
}

inline
const std::string& 
Message::function() const 
{ 
   return function_; 
}

inline
Message::Message(const Parameter& id, const std::string& func)
: instance_(id), function_(func) 
{
}

inline
Message& 
Message::result(const Parameter& arg)
{
   args_.clear();
   args_.push_back(arg);   
   return *this;
}

extern void     connect (cchar* read_pipe, cchar* write_pipe);
extern void       write (const std::string&);
extern std::string read ();
extern std::string pack (
      const Parameter&, 
      cchar*, 
      const Parameter& =Parameter::none(), 
      const Parameter& =Parameter::none(), 
      const Parameter& =Parameter::none());

extern Message unpack(const std::string&);

}

#endif // RMI_CONNECT_H

