#ifndef RMI_PARAMETER_H
#define RMI_PARAMETER_H

#include <string>
#include <rmi/convert.h>

namespace RMI {

class Parameter
{
   public:

   template<class Type>
   Parameter(const Type& value) : value_(convert(value)) {}

   // Optimization it the type is std::string.
   Parameter(const std::string& value) : value_(value) {}

   const std::string& string() const { return value_; }

   // Implicit conversions back to the original types.
   // The first two conversions are used only by the server that needs
   // to convert received parameters into actual pointers.

   template<class Type> operator Type*() const;
   template<class Type> operator const Type*() const;
   template<class Type> operator Type() const;

   bool operator==(const Parameter&) const;
   bool operator!=(const Parameter&) const;

   static const Parameter& none();

   private:

   std::string value_;
};

template<class Type>
inline
Parameter::operator Type*() const
{
   printf("Parameter::operator Type*()\n");
   return (Type*) convert<void*>(value_);
}

template<class Type>
inline
Parameter::operator const Type*() const
{
   printf("Parameter::operator const Type*()\n");
   return (Type*) convert<void*>(value_);
}

template<class Type>
inline
Parameter::operator Type() const
{
   printf("Parameter::operator Type()\n");
   return convert<Type>(value_);
}

inline
bool 
Parameter::operator==(const Parameter& parameter) const
{
   return value_ == parameter.value_;
}

inline
bool 
Parameter::operator!=(const Parameter& parameter) const
{
   return value_ != parameter.value_;
}

}

#endif // RMI_PARAMETER_H

