#ifndef NAMED_PIPE_H
#define NAMED_PIPE_H

#include<stdlib.h>
#include<stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

typedef const char cchar;

namespace IO {

   class Access
   {
      public:

      static const Access&  read();
      static const Access& write();

      Access operator+(const Access& type) const
      {
         return Access(id_ + type.id_);
      };
      bool operator==(const Access& type) const
      {
         return id_ == type.id_;
      };
      bool operator!=(const Access& type) const
      {
         return id_ != type.id_;
      };
      bool  is_read() const { return id_ & read(). id_; };
      bool is_write() const { return id_ & write().id_; };

      private:

      Access(int id) : id_(id) {}

      int id_;
   };

class Pipe
{
   public:

  ~Pipe();
   Pipe();
   Pipe(cchar* name, const Access&);

   void  open (cchar* name, const Access&);
   int   read (void*, int);
   void write (const void*, int);

   private:
   
   Pipe(const Pipe&);
   Pipe& operator=(const Pipe&);

   int fd_;
};

}

#endif // NAMED_PIPE_H
