#ifndef RMI_CLIENT_H
#define RMI_CLIENT_H

#include <rmi/connect.h>
#include <rmi/parameter.h>

namespace RMI {

using std::string;

class RMIB;

class Call
{
   public:

   Call(const RMIB& instance,
        const char* function, 
        const Parameter& arg1 =Parameter::none(), 
        const Parameter& arg2 =Parameter::none(), 
        const Parameter& arg3 =Parameter::none());

   // Converter is needed to be able to simply write
   //    Foo foo() { return Call(); } 
   // that will automatically convert Call into a proper type.
   
   template<class Type> operator Type() const { return result_; }

   const Parameter& result() const { return result_; }

   private:
   
   Parameter result_;
};

class RMIB // RMI Base
{
   public:

   virtual ~RMIB() {}

   RMIB(const Call&);
   
   // Converter is used when a RMIB-derived instance is passed as an argument
   // like in Foo::operator=(const Foo&).
   // It has precedence over template<class T> Parameter(const T&).
   // Using the converter is quicker as Param::Param(const T&) uses
   // convert() that creates and returns std::string that in turn is used
   // to create a new Parameter instance. It's way cheaper to simply return 
   // areference to id_ that we already have stored.

   operator const Parameter&() const; 
   
   RMIB&    name (const string&); // Give a name to the remote object.
   RMIB&    bind (bool =true);    // Bind/unbind the proxy-obj. link. 
   bool is_bound () const;

   static const RMIB& null();

   private:

   RMIB(); // Used to create null() reference.

   Parameter id_; // RMI instance ID.
   bool   bound_; // If 'true', deletes the remote obj. when goes out of scope.
};

/* If we did not use "RMIB-to-const Parameter&" converter, that function
 * would be used every time a proxy was passed as an argument. That function
 * will compile only for the classes derived from RMIB. Rogue classes would
 * not compile.
template<class T>
inline
string
convert(const T& instance)
{
   printf("!!!!!!! template<> string convert(const T&)\n");
   return instance.id().string();
}
*/

}

#endif // RMI_CLIENT_H
