#ifndef RMI_SERVER_H
#define RMI_SERVER_H

#include <map>
#include <rmi/String.h>
#include <rmi/convert.h>
#include <rmi/connect.h>

// One Server:   RMI::Foo* foo = new RMI::Foo();
// Many Servers: RMI::Foo* foo = new(connection) RMI::Foo();

namespace RMI {

class Server
{
   public:

   Server();

   void run() const;

   private:

   typedef const Message& (*Function)(Message&);
   typedef std::map<String, Function> Functions;

   static Functions functions_;
};

typedef const void* cvoidp;

// Conversion functions needed to convert pointers/references returned from 
// the real functions to strings for them to be sent to the client as a reference.

template<class T>
inline
std::string
convert(T* instance)
{
   printf("!!! template<> string convert(T*)\n");
   return convert(cvoidp(instance));
}

template<class T>
inline
std::string
convert(const T* instance)
{
   printf("!!! template<> string convert(const T*)\n");
   return convert(cvoidp(instance));
}

template<class T>
inline
std::string
convert(const T& instance)
{
   printf("!!!!!!!!!!!!!!!!!!!!!!!! template<> string convert(const T&)\n");
   return convert(cvoidp(&instance));
}

}

#endif // RMI_SERVER_H
