#include <rmi/pipe.h>
     #include <sys/types.h>
     #include <unistd.h>
     #include <fcntl.h>


const IO::Access& IO::Access:: read() { static Access a(0x01); return a; }
const IO::Access& IO::Access::write() { static Access a(0x02); return a; }

IO::Pipe::~Pipe()
{
   if (fd_ != -1) ::close(fd_);
}

IO::Pipe::Pipe() : fd_(-1)
{
}

IO::Pipe::Pipe(cchar* name, const Access& access)
{
   open(name, access);
}

void
IO::Pipe::open(cchar* name, const Access& access)
{
   int    result = mknod(name, S_IFIFO | 0666, 0);
//   int open_mode = access.is_write() ? O_WRONLY : (O_RDONLY | O_NDELAY);
   int open_mode = O_RDWR;
   
   if (result != 0 && errno != EEXIST)
   {
      perror("Cannot create named pipe");
      exit(0);
   }
   fd_ = ::open(name, open_mode);
}

int   
IO::Pipe::read(void* data, int size)
{
   fcntl(fd_, ~O_NDELAY);
   int num_bytes = ::read(fd_, data, size);
   printf("Read:  <%s>\n", data);
   return num_bytes;
}

void 
IO::Pipe::write(const void* data, int size)
{
   printf("Write: <%s>\n", data);
   ::write(fd_, data, size);
}
