#include <rmi/rmi.h>

using namespace RMI;
using std::string;

typedef const void*       const_void_ptr;
typedef const Parameter& const_param_ref;

static
Parameter 
call(
   const RMIB&  instance,
   const char*      func, 
   const Parameter& arg1, 
   const Parameter& arg2, 
   const Parameter& arg3)
{
   printf("RMI::Call::Call(%s, %s, %s, %s, %s)\n", 
          const_param_ref(instance).string().c_str(),
          func,  
          arg1.string().c_str(), 
          arg2.string().c_str(), 
          arg3.string().c_str());

   write(pack(instance, func, arg1, arg2, arg3));

   return unpack(read()).arg(0);
}

RMI::Call::Call(
   const RMIB&  instance,
   cchar*           func, 
   const Parameter& arg1, 
   const Parameter& arg2, 
   const Parameter& arg3)
:  result_(call(instance, func, arg1, arg2, arg3))
{
}

RMI::RMIB::RMIB() : id_(const_void_ptr(0)), bound_(false)
{ 
}

RMI::RMIB::RMIB(const Call& call) : id_(call.result()), bound_(false) 
{
}

const RMIB& 
RMI::RMIB::null() 
{ 
   static RMIB instance;
   return instance; 
}

RMI::RMIB::operator const Parameter&() const 
{ 
   printf("RMIB::operator const Parameter&() const\n");
   return id_; 
}
   
RMIB& 
RMI::RMIB::name(const string& new_name) 
{
   return (Call(*this, "name", new_name), *this);
}

RMIB& 
RMI::RMIB::bind(bool state) 
{ 
   return (bound_ = state, *this); 
}

bool 
RMI::RMIB::is_bound() const 
{ 
   return bound_; 
}
