#include <iostream>
#include <rmi/connect.h>
#include <rmi/rmi.h>

struct Foo {};

namespace RMI {

struct Point : public RMIB
{
  ~Point();
   Point(int, int);
   Point(const Point&);
   Point& operator=(const Point&);
   Point(const Call& call) : RMIB(call) {} // Mandatory constructor.

   static std::string info(const Point&);
};

Point::~Point()
{
   if (is_bound()) Call(*this, "Point::~Point()");
}

Point::Point(int arg1, int arg2)
: RMIB(Call(null(), "Point::Point(int, int)", arg1, arg2))
{
   bind();
}

Point::Point(const Point& arg1) 
:  RMIB(Call(null(), "Point::Point(const Point&)", arg1))
{
   bind();
}

Point&
Point::operator=(const Point& arg1) 
{
   Call(*this, "Point& Point::operator=(const Point&)", arg1);
   return *this;
}

std::string
Point::info(const Point& arg1) 
{
   return Call(null(), "std::string Point::info(const Point&)", arg1);
}

struct Widget : public RMIB
{
  ~Widget();
   Widget(const std::string&);
   Widget(const Widget&);
   Widget& operator=(const Widget&);
   Widget(const Call& call) : RMIB(call) {} // Mandatory constructor.
   
   std::string name() const;
};

Widget::~Widget()
{
   if (is_bound()) Call(*this, "Widget::~Widget()");
}

Widget::Widget(const std::string& arg1)
: RMIB(Call(null(), "Widget::Widget(const std::string&)", arg1))
{
   bind();
}

Widget::Widget(const Widget& arg1) 
:  RMIB(Call(null(), "Widget::Widget(const Widget&)", arg1))
{
   bind();
}

Widget&
Widget::operator=(const Widget& arg1) 
{
   Call(*this, "Widget& Widget::operator=(const Widget&)", arg1);
   return *this;
}

std::string
Widget::name() const
{
   return Call(*this, "const std::string& Widget::name() const");
}

struct Circle : public Widget
{
   // No destructor.
   
   Circle(const std::string&, const RMI::Point&, int);
   Circle(const Circle&);
   Circle& operator=(const Circle&);
   Circle(const Call& call) : Widget(call) {} // Mandatory constructor.

   void center(const Point&);
   Point center() const;
};

Circle::Circle(const std::string& arg1, const Point& arg2, int arg3)
: Widget(Call(null(), "Circle::Circle(const std::string&, const Point&, int)", arg1, arg2, arg3))
{
   bind();
}

Circle::Circle(const Circle& arg1) 
:  Widget(Call(null(), "Circle::Circle(const Circle&)", arg1))
{
   bind();
}

Circle&
Circle::operator=(const Circle& arg1) 
{
   Call(*this, "Circle& Circle::operator=(const Circle&)", arg1);
   return *this;
}

void
Circle::center(const Point& arg1) 
{
   Call(*this, "void Circle::center(const Point&)", arg1);
}

Point
Circle::center() const
{
   return Call(*this, "const Point& Circle::center() const");
}

} // End of RMI namespace.

using namespace RMI;

#define TEST(cmd) printf("*** " #cmd "\n"); cmd

int
main()
{
   TEST(RMI::connect("/tmp/write", "/tmp/read"));

   TEST(Point point1(11, 22));
   TEST(Point point2(33, 44));
   TEST(std::string info = Point::info(point1));

   point1.bind();

   // Test copying.
   TEST(point1 = point2);
   TEST(std::string copied_info = Point::info(point1));

   TEST(Widget widget("widget"));
   TEST(Circle circle("circle", point1, 5));
   TEST(Widget* circlep = new Circle("circlep", point2, 9));

   TEST(std::string wname = widget.name());
   TEST(std::string cname = circle.name());
   TEST(std::string pname = circlep->name());

   printf("*** %s/%s/%s.\n", wname.c_str(), cname.c_str(), pname.c_str());

   TEST(circle.center(point2));
   TEST(Point new_center = circle.center());
   TEST(std::string new_center_info = Point::info(new_center));

   printf("*** new center: %s.\n", new_center_info.c_str());
}

