#ifndef FOO_H
#define FOO_H

#include <stdio.h>
#include <string>

using std::string;

class Point
{
   public:
   
   // The class does not have default constructor.
   // Copy ctor and op=() are compiler-generated.

  ~Point() { printf("*** Point::~Point()\n"); }
   // Test passing by-value (x, y).
   Point(int x, int y);
   // Test static.
   // Test passing objects by-reference.
   // Test returning an object by-value (of type std::string).
   static std::string info(const Point&);

   private:
   
   int x_;
   int y_;
};

class Widget
{
   public:
   
   virtual ~Widget() { printf("*** Widget::~Widget()\n"); }
   // Test passing by-value (name).
   Widget(const string& name);

   // Test returning an object by-value.
   const std::string& name() const;
   // Test static.
   static string print(const Widget&);
   // Test virtual
   virtual string info() const;

   private:
   
   string name_;
};

class Circle : public Widget
{
   typedef Widget super;

   public:

  ~Circle() { printf("*** Circle::~Circle()\n"); }
   // Test passing by-value (name) and by-reference (center).
   Circle(const std::string& name, const Point& center, int radius); 
   // Test virtual
   virtual string info() const;
   // 
   void center(const Point&);
   // Test returning by-const-reference.
   const Point& center() const;
   // Test returning by-non-const-reference.
   Point& center();
   
   private:
   
   Point center_;
   int   radius_;
};

inline
Widget::Widget(const std::string& name) : name_(name)
{
   printf("*** Widget::Widget(name=%s)\n", name.c_str());
}

inline
Point::Point(int x, int y) : x_(x), y_(y) 
{
   printf("*** Point::Point(x=%d, y=%d)\n", x, y);
}

inline
Circle::Circle(const std::string& name, const Point& center, int radius)
: Widget(name), center_(center), radius_(radius)
{
   printf("*** Circle::Circle(name=%s, center=%s, radius=%d)\n", name.c_str(), Point::info(center).c_str(), radius);
}

inline
std::string
Widget::info() const 
{ 
   return name_; 
}

inline
std::string
Point::info(const Point& point)
{ 
   return String::format("%d, %d", point.x_, point.y_);
}

inline
std::string
Circle::info() const 
{ 
   return String::format("%s, center=%s, radius=%d", 
      super::info().c_str(), Point::info(center_).c_str(), radius_);
}

inline
const string&
Widget::name() const
{
   return name_;
}

inline
void
Circle::center(const Point& point)
{
   printf("*** void Circle::center(const Point&) const\n");
   center_ = point;
}

inline
const Point&
Circle::center() const
{
   printf("*** const Point& Circle::center() const\n");
   return center_;
}

inline
Point&
Circle::center()
{
   printf("*** Point& Circle::center()\n");
   return center_;
}









#endif // FOO_H


