#include <rmi/pipe.h>
#include <rmi/convert.h>
#include <rmi/server.h>
#include "Foo.h"

RMI::Server::Functions RMI::Server::functions_;

namespace RMI {

const Message&
func001(Message& msg)
{
   Point* instance = msg.instance();
   return (delete instance, msg);
}

const Message&
func002(Message& msg)
{
   Widget* instance = msg.instance();
   return (delete instance, msg);
}

const Message&
func003(Message& msg)
{
   int arg0 = msg.arg(0);
   int arg1 = msg.arg(1);

   return msg.result(new Point(arg0, arg1));
}

const Message&
func004(Message& msg)
{
   std::string arg0 = msg.arg(0);

   return msg.result(new Widget(arg0));
}

const Message&
func005(Message& msg)
{
   std::string  arg0 = msg.arg(0);
   const Point* arg1 = msg.arg(1);
   int          arg2 = msg.arg(2);

   return msg.result(new Circle(arg0, *arg1, arg2));
}

const Message&
func006(Message& msg)
{
   Point* instance = msg.instance();
   Point*     arg0 = msg.arg(0);
   
   return msg.result(instance->info(*arg0));
}

const Message&
func007(Message& msg)
{
   Widget* instance = msg.instance();
   
   return msg.result(instance->info());
}

const Message&
func008(Message& msg)
{
   Circle* instance = msg.instance();
   
   return msg.result(instance->info());
}

const Message&
func009(Message& msg)
{
   Point* arg0 = msg.arg(0);

   return msg.result(new Point(*arg0));
}

const Message&
func010(Message& msg)
{
   Point* instance = msg.instance();
   Point*     arg0 = msg.arg(0);

   return msg.result(instance->operator=(*arg0));
}

const Message&
func011(Message& msg)
{
   Widget* arg0 = msg.arg(0);

   return msg.result(new Widget(*arg0));
}

const Message&
func012(Message& msg)
{
   Widget* instance = msg.instance();
   Widget*     arg0 = msg.arg(0);

   return msg.result(instance->operator=(*arg0));
}

const Message&
func013(Message& msg)
{
   Circle* arg0 = msg.arg(0);

   return msg.result(new Circle(*arg0));
}

const Message&
func014(Message& msg)
{
   Circle* instance = msg.instance();
   Circle*     arg0 = msg.arg(0);

   return msg.result(instance->operator=(*arg0));
}

const Message&
func015(Message& msg)
{
   Circle* instance = msg.instance();

   return msg.result(instance->name());
}

const Message&
func016(Message& msg)
{
   Circle* instance = msg.instance();
   Point*      arg0 = msg.arg(0);

   return (instance->center(*arg0), msg);
}

const Message&
func017(Message& msg)
{
   Circle* instance = msg.instance();

   return msg.result(instance->center());
}

}

///////////////////////////////////////////////////////////////

RMI::Server::Server()
{
   // Destructors.

   functions_["Point::~Point()"]   = func001;
   functions_["Widget::~Widget()"] = func002;
   // Circle. No destructor for derived classes. Handled by the virt. base dtor.

   // Constructors.

   functions_["Point::Point(int, int)"]                                = func003;
   functions_["Widget::Widget(const std::string&)"]                    = func004;
   functions_["Circle::Circle(const std::string&, const Point&, int)"] = func005;

   // info()

   functions_["std::string Point::info(const Point&)"] = func006;
   functions_["std::string Widget::info() const"]      = func007;
   functions_["std::string Circle::info() const"]      = func008;

   // Always generated.

   functions_["Point::Point(const Point&)"]               = func009;
   functions_["Point& Point::operator=(const Point&)"]    = func010;
   functions_["Widget::Widget(const Widget&)"]            = func011;
   functions_["Widget& Widget::operator=(const Widget&)"] = func012;
   functions_["Circle::Circle(const Circle&)"]            = func013;
   functions_["Circle& Circle::operator=(const Circle&)"] = func014;

   functions_["const std::string& Widget::name() const"] = func015;
   functions_["void Circle::center(const Point&)"]       = func016;
   functions_["const Point& Circle::center() const"]     = func017;
}

void
RMI::Server::run() const
{
   RMI::connect("/tmp/read", "/tmp/write");

   for (;;)
   {
      Message            msg = RMI::unpack(RMI::read());
      Functions::iterator it = functions_.find(msg.function());

      if (it == functions_.end())
      {
         printf("Not found: %s.\n", msg.function().c_str());
      }
      else 
      {
         RMI::write(it->second(msg).pack());
      }
   }
}

int
main()
{
   RMI::Server().run();
}
